function [frame_count,status_fig] = GMV_Sweetwater()
%
%###################################################################
%
% Name: GMV_1C.m - a script to generate Ground Motion Visualization (GMV) movies
%
% Copyright (C) 2011  Product Team, IRIS Data Management Center 
%
%    This is a free software; you can redistribute it and/or modify
%    it under the terms of the GNU Lesser General Public License as
%    published by the Free Software Foundation; either version 3 of the
%    License, or (at your option) any later version.
%    
%    This software is distributed in the hope that it will be useful, but
%    WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%    Lesser General Public License (GNU-LGPL) for more details.  The
%    GNU-LGPL and further information can be found here:
%    http://www.gnu.org/
%
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Description:
%
%    Script reads through a series of SAC files in a directory and an event
%    file in the same directory and creates a Ground Motion Visualization
%    movie based on the parameters provided in the event file.
%
% Input:
%
%    Script expects the dataPath to contain an event file with necessary
%    parameters as described below and the associated SAC file.
%
%    ../event.asc
%   
%    with each line containing KEY,VALUE pairs with "|" as separator:
%
%    KEY|VALUE
%
%    ID| GMV ID will be used as part of the movie file name
%    TITLE| a string title for the GMV
%    WIN_START| start of the GMV time window as YYYY-MM-DD HH24:MI:SS UTC
%    WIN_END| end of the GMV time window as YYYY-MM-DD HH24:MI:SS UTC
%    WIN_STEP| create a frame for every WIN_STEP seconds of real time
%    EVENT_LAT| (optional) event latitude, if not provided, arrival times
%                          will not be marked on the reference seismogram
%                          -90/90
%    EVENT_LON| (optional) event longitude, if not provided, arrival times
%                          will not be marked on the reference seismogram
%                          -180/180.
%    EVENT_TIME| (optional) event as YYYY-MM-DD HH24:MI:SS UTC. If not
%                          provided, arrival times will not be marked on 
%                          the reference seismogram
%    SHOW_GC| show the great circle path on GMV, 1=yes, 0=no
%    LL_LAT| lower left latitude of the animation window (-90/90)
%    LL_LON| lower left longitude of the animation window (-180/180)
%    UR_LAT| upper right latitude of the animation window (-90/90)
%    UR_LON| upper right longitude of the animation window (-180/180)
%    WIDTH| animation window width in pixels (800 maximum)
%    MAGNIFICATION | trace magnification. Higher trace magnification 
%                    highlights weaker arrivals at the expense of saturating 
%                    colors for the stronger arrivals. The default value of 10 
%                    is suitable for most events. For close events, a magnification 
%                    factor of 5 or less would place less emphasis on unwanted 
%                    near-source arrivals.
%    CHANNEL|channel to use for GMV. The format is TECH_CHAN where TECH is
%                     the technology (SEIS for seismic or INFRA  
%                     for infrasound and CHAN is the channel name)
%    LP| (optional) low-pass filter corner frequency (HZ)
%    HP| (optional) high-pass filter corner frequency (HZ)    
%
% example:
%
%    ID|123
%    TITLE|GULF OF CALIFORNIA, M=6.7, 2010-10-21, 0.001-0.01 Hz
%    WIN_START|2010-10-21 17:52:42
%    WIN_END|2010-10-21 18:26:42
%    WIN_STEP|4
%    EVENT_LAT|24.69
%    EVENT_LON|-109.159
%    EVENT_TIME|2010-10-21 17:53:13
%    SHOW_GC|1
%    MAGNIFICATION|10
%    LL_LAT|19.97
%    LL_LON|-125.86
%    UR_LAT|50.0
%    UR_LON|-65.5
%    WIDTH|800
%    CHANNEL|SEIS_LHZ
%    LP|0.01
%    HP|0.001      
%   
% History:
%
%    2012-01-12 (MB): user can now set the magnification factor via the control file
%    2012-01-09 (MB): added Debi Kilb's code to list the filter and time step
%    2011-07-12 (MB): added suggestion by Debi Kilb to correctlt compute movieT for vert
%               short movies. The correction include:
%                   1) Initialize the movieT parameter to be an array and not a matrix by adding in that '1' when you use zeros
%                   2) use 'double(p)' so it doesn't force the value to be an integer.
%
%    2011-07-05 IRIS DMC Product Team (MB): introduced the sacFileExtension parameter
%    2011-06-16 IRIS DMC Product Team (MB): revised for public distribution
%    2009 Bob Woodward (IRIS): Original version 
%
%
%
% 
% configuration parameters
%
%
% files and paths
%
shoreline             = 'world_coast.dat';              % shoreline
boundaries            = 'world_boundaries.dat';         % world political boundaries
political             = 'US_political.dat';             % US political boundaries
topoFileInterval      = 5;                              % 5 seconds topo file resolution
topoFile              = 'topohires.mat';                % topography file
eventFile             = 'event.asc';                    % default event file name
dataPath              = '../data';                      % data directory
parFile               = strcat(dataPath,'/',eventFile); % parameter file
moviePath             = '../movie';                     % movie output directory
irisLogo              = '../image/IRIS_Logo_Url.png';   % IRIS logo file name

path(path,'../lib/');      % library path
path(path,'../mat/');      % matlab data files path
path(path,'../matTaup/');  % matTaup path

%
% default parameter values
%
moviePrefix           = 'GMV'; % movie faile prefix

irisLogoRight         = 0.0;  % IRIS logo position, right
irisLogoUp            = 0.005; % IRIS logo position, up
irisLogoWidth         = 0.45; % IRIS logo width
irisLogoHeight        = 0.050;% IRIS logo height
irisLogoPosition      = [irisLogoRight,irisLogoUp,irisLogoWidth,irisLogoHeight]; % lower left,size

pSlowness             = 7.2;            % P slowness (s/degree);  a P slowness of ~ 7.2 s/deg (should be conservative out to 125 deg)
rSlowness             = 28.5;           % Rayleigh slowness (s/degree); originally 29.835 but using slowness of 28.5 to provides a better phase display
iSlowness             = 111.195 / 0.29; % Infrasound slowness (s/degree)

magnification         = 10;         % each station is individually scaled to between [-magnification and * magnification]

phaseList             = 'P,Pn,Pg,Pdiff,pPdiff,sPdiff,PcP,PP,PP,PS,PKP,SKS,S,Sn,Sg,Sdiff,pSdiff,sSdiff,SP,ScS,SS,SKS'; %list of seismic phases to calculate travel times for
phaseSpacing          = 200;        % number of seconds between two consecutive phases to avoid over plots
    
eventDepth            = 0;          % event depth
screenWidth           = 800;        % default movie width in pixels
maxScreenHeight       = 800;        % maximum movie height in pixels
frameFlushInterval    = 50;         % every frameFlushIntervals will be flushed out (improve speed) 
headerFontSize        = 14;         % movie header font size
circleThickness       = 3;          % thickness of the circle around the reference station
circleColor           = 'yellow';   % color of the circle around the reference station
markerEdgeColor       = 'k';        % marker edge color. The color of the marker or the 
                                    % edge color for filled markers (circle, square, diamond, pentagram, 
                                    % hexagram, and the four triangles). ColorSpec  defines the color to use. 
                                    % none specifies no color, which makes nonfilled markers invisible. auto sets 
                                    % MarkerEdgeColor  to the same color as the CData property.
markerSize            = 55;         % circle marker size for the stations
refMarkerSize         = 180;        % circle marker size for the reference station
xTick                 = 86400;      % seismogram tick spacing in seconds
cmap2                 = [1 1 1; 1 1 1; 1 1 1; 0.9 0.9 0.9]; % alternate color map for shading the US for 'topolores.mat'
ftype                 = 'NONE';     % filter type
filter                = [0 0];      % corner filter frequencies

sacFileExtension      = '.SAC';     % script expects waveform data to be in SAC format and "sacFileExtension"
                                    % tells the script what file extension to look for when reading the data
                                    % note that script expects a file name that contains channel name and has
                                    % sacFileExtension as the extension. If your file names are structured differently, you need to change
                                    % the part of the code under " read seismograms, filter and resample"

debug                 = false;      % turn informative messages on(true) or off (false)
%
%###################################################################
%
% read in movie parameters from the parameter file
%
[flag value]          = get_custom_par('ID',parFile);
if (flag > 0)
  movieId = value;
  if(debug)
    fprintf('\n[INFO] ID:%s\n',movieId);
  end;
end;

[flag value]          = get_custom_par('TITLE',parFile); 
if (flag > 0)
  plotTitle = value;
  if(debug)
    fprintf('\n[INFO] plotTitle:%s\n',plotTitle);
  end;
end;

filterString = ['; no filter applied'];

[flag value]          = get_custom_par('HP',parFile);
hp = 0;
if (flag > 0)
  flow = str2double(value);
  filter = [flow;flow];
  hp = 1;
  ftype = 'HP';
  filterString = [' Filter (Hz): ',num2str(flow),' high-pass'];
  if(debug)
    fprintf('\n[INFO] %s filter frequency:%d\n',ftype,flow);
  end;
end;

[flag value]          = get_custom_par('LP',parFile);
lp=0;
if (flag > 0)
  fhigh = str2double(value);
  filter = [fhigh;fhigh];
  lp = 1;
  ftype = 'LP';
  filterString = [' Filter (Hz): ',num2str(fhigh),' low-pass'];
  if(debug)
    fprintf('\n[INFO] %s filter frequency:%d\n',ftype,fhigh);
  end;
end;

if(hp==1 && lp==1)
   ftype = 'BP';
   filter = [flow;fhigh];
   filterString = [' Filter (Hz): ',num2str(flow),' to ',num2str(fhigh)];
  if(debug)
    fprintf('\n[INFO] %s filter frequency:%d-%d\n',ftype,flow,fhigh);
  end;
end;
 
isEventSet = false;
[flag value]          = get_custom_par('EVENT_LAT',parFile); 
if (flag >0 )
  eventLat = str2double(value);
  isEventSet = true;
  [flag value]          = get_custom_par('EVENT_LON',parFile);
  if (flag > 0)
    eventLon = str2double(value);
  end;

  if(debug)
    fprintf('\n[INFO] event lat/lon:%f to %f\n',eventLat,eventLon);
  end;
  [flag value]     = get_custom_par('EVENT_TIME',parFile);

  eventTime  = value;

  if(debug)
    fprintf('\n[INFO] event:<%s>\n',eventTime);
  end;

  [value1 value2] = strread(eventTime,'%s%s','delimiter',' ');
  eventDate  = value1{1};
  eventTime  = value2{1};
  if(debug)
    fprintf('\n[INFO] event start:%s T %s\n',eventDate,eventTime);
  end;

  [value1 value2 value3] = strread(eventDate,'%s%s%s','delimiter','-');
  eventYear  = str2double(value1{1});
  eventMonth = str2double(value2{1});
  eventDay   = str2double(value3{1});
  if(debug)
    fprintf('\n[INFO] event Date:%d/%d/%d\n',eventYear,eventMonth,eventDay);
  end;

  [value1 value2 value3] = strread(eventTime,'%s%s%s','delimiter',':');
  eventHour  = str2double(value1{1});
  eventMinute = str2double(value2{1});
  eventSecond = str2double(value3{1});
  if(debug)
    fprintf('\n[INFO] event Time:%d-%d-%d\n',eventHour,eventMinute,eventSecond);
  end;

  eventJday        = jday(eventYear, eventMonth, eventDay); % get julian day
  eventEpoch       = epoch(eventYear,eventJday,eventHour,eventMinute,eventSecond); %now, translate all events times into epochal
  if(debug)
    fprintf('\n[INFO] event Start Jday,epoch:%d,%d\n',eventJday,eventEpoch);
  end;
end;

[flag value]          = get_custom_par('LL_LAT',parFile); % map extent to the south
if (flag > 0)
  minLatIn = str2double(value);
  minLat = ((asinh(tan(minLatIn*pi/180.))* 180.0 / pi));
end;

[flag value]          = get_custom_par('UR_LAT',parFile); % map extent to the north
if (flag > 0)
  maxLatIn = str2double(value);
  maxLat = ((asinh(tan(maxLatIn*pi/180.))* 180.0 / pi));
end;

if(debug)
  fprintf('\n[INFO] min,max lat:%d,%d\n',minLat,maxLat);
end;

[flag value]          = get_custom_par('LL_LON',parFile); % map extent to the west
if (flag > 0)
  minLonIn = str2double(value);
  minLon = minLonIn;
end;

[flag value]          = get_custom_par('UR_LON',parFile); % map extent to the east
if (flag > 0)
  maxLonIn = str2double(value);
  maxLon = maxLonIn;
end;

centerLon             = (minLon+maxLon)/2.0; % set to -999 to center on the array
minLon = minLon - centerLon;
maxLon = maxLon - centerLon;

[flag value]          = get_custom_par('WIDTH',parFile); % map width
if (flag > 0)
  screenWidth = floor(str2double(value));
  if(mod(screenWidth,2) == 1) % make sure width is even so ffmpeg has no problem!
     screenWidth = screenWidth + 1;
   end;
end;

if(debug)
  fprintf('\n[INFO] min,max,center lon:%d,%d,%d\n',minLon,maxLon,centerLon);
end;

minViewLat            = minLat; % minimum latitude of the view port
maxViewLat            = maxLat; % maximum latitude of the view port
dlat = abs(maxViewLat - minViewLat);

minViewLon            = minLon; % minimum longitude of the view port
maxViewLon            = maxLon; % maximum longitude of the view port
dlon = abs(maxViewLon - minViewLon);
timeHorizontalOffset  = 0;  % degrees longitude offset of the time stamp from the lower left corner of the map

if(debug)
   fprintf('\n[INFO] dlat, dlon:%d,%d\n',dlat,dlon);
end; % if debug
%
% adjust video dimensions based on the selected view port
%
screenHeight = floor(screenWidth * (dlat/dlon) * 4.0/3.0);
if(mod(screenHeight,2) == 1) % make sure width is height so ffmpeg has no problem!
   screenHeight = screenHeight + 1;
end;
if (screenHeight > maxScreenHeight)
   screenWidth = ceil(screenWidth * maxScreenHeight/screenHeight); % adjust the screen height
   screenHeight = maxScreenHeight;
end;
if(mod(screenWidth,2) == 1)% make sure width is even so ffmpeg has no problem!
   screenWidth = screenWidth + 1;
end;

timeVerticalOffset    = -600*dlat/screenHeight/32.0; % degrees latitude offset of the time stamp from the lower left corner of the map

if (debug)
   fprintf('\n[INFO] screen size:%d,%d\n',screenWidth,screenHeight);
end; %if debug
   
showGCPath            = false;      % show the great circle path?
[flag value]     = get_custom_par('SHOW_GC',parFile);
if(flag > 0 && str2double(value) == 1)
   showGCPath=true;
end;

[flag value]     = get_custom_par('MAGNIFICATION',parFile); 
if(flag > 0)
   magnification=str2double(value);
end;

[flag movieStart]     = get_custom_par('WIN_START',parFile); % movie start time
if(~isEventSet)
   eventTime = movieStart;
end;

[flag movieEnd]       = get_custom_par('WIN_END',parFile); % movie start time
[flag value]     = get_custom_par('WIN_STEP',parFile); % movie time step, in s

movieTStep  = str2double(value);
 dtString = [' Time Step: ',num2str(movieTStep),'s '];

if(debug)
  fprintf('\n[INFO] movie:<%s> to <%s>\n',movieStart,movieEnd);
end;

[value1 value2] = strread(movieStart,'%s%s','delimiter',' ');
movieStartDate  = value1{1};
movieStartTime  = value2{1};
if(debug)
  fprintf('\n[INFO] movie start:%s T %s\n',movieStartDate,movieStartTime);
end;

[value1 value2 value3] = strread(movieStartDate,'%s%s%s','delimiter','-');
movieStartYear  = str2double(value1{1});
movieStartMonth = str2double(value2{1});
movieStartDay   = str2double(value3{1}); 
if(debug)
  fprintf('\n[INFO] movie Date:%d/%d/%d\n',movieStartYear,movieStartMonth,movieStartDay);
end;

[value1 value2 value3] = strread(movieStartTime,'%s%s%s','delimiter',':');
movieStartHour  = str2double(value1{1});
movieStartMinute = str2double(value2{1});
movieStartSecond = str2double(value3{1}); 
if(debug)
  fprintf('\n[INFO] movie Time:%d-%d-%d\n',movieStartHour,movieStartMinute,movieStartSecond);
end;

movieStartJday        = jday(movieStartYear, movieStartMonth, movieStartDay); % get julian day
movieStartEpoch       = epoch(movieStartYear,movieStartJday,movieStartHour,movieStartMinute,movieStartSecond); %now, translate all events times into epochal
if(debug)
  fprintf('\n[INFO] movie Start Jday,epoch:%d,%d\n',movieStartJday,movieStartEpoch);
end;

[value1 value2] = strread(movieEnd,'%s%s','delimiter',' ');
movieEndDate  = value1{1};
movieEndTime  = value2{1};
if(debug)
  fprintf('\n[INFO] movie start:%s T %s\n',movieEndDate,movieEndTime);
end;

[value1 value2 value3] = strread(movieEndDate,'%s%s%s','delimiter','-');
movieEndYear  = str2double(value1{1});
movieEndMonth = str2double(value2{1});
movieEndDay   = str2double(value3{1});
if(debug)
  fprintf('\n[INFO] movie Date:%d/%d/%d\n',movieEndYear,movieEndMonth,movieEndDay);
end;

[value1 value2 value3] = strread(movieEndTime,'%s%s%s','delimiter',':');
movieEndHour  = str2double(value1{1});
movieEndMinute = str2double(value2{1});
movieEndSecond = str2double(value3{1});
if(debug)
  fprintf('\n[INFO] movie Time:%d-%d-%d\n',movieEndHour,movieEndMinute,movieEndSecond);
end;

movieEndJday        = jday(movieEndYear, movieEndMonth, movieEndDay); % get julian day
movieEndEpoch       = epoch(movieEndYear,movieEndJday,movieEndHour,movieEndMinute,movieEndSecond); %now, translate all events times into epochal
if(debug)
  fprintf('\n[INFO] movie End Jday,epoch:%d,%d\n',movieEndJday,movieEndEpoch);
end;

%
% calculate movie length and time label spacing
%
movieLength = movieEndEpoch - movieStartEpoch;
if(movieLength < 2.2*xTick)
   xTick = 86400;
end;
if(movieLength < 2.2*xTick)
   xTick = 44800;
end;
if(movieLength < 2.2*xTick)
   xTick = 22400;
end;
if(movieLength < 2.2*xTick)
   xTick = 11200;
end;
if(movieLength < 2.2*xTick)
   xTick = 9600;
end;
if(movieLength < 2.2*xTick)
   xTick = 4800;
end;
if(movieLength < 2.2*xTick)
   xTick = 2400;
end;
if(movieLength < 2.2*xTick)
   xTick = 1200;
end;
if(movieLength < 2.2*xTick)
   xTick = 600;
end;
if(movieLength < 2.2*xTick)
   xTick = 300;
end;
if(movieLength < 2.2*xTick)
   xTick = 180;
end;
if(movieLength < 2.2*xTick)
   xTick = 120;
end;
if(movieLength < 2.2*xTick)
   xTick = 60;
end;

if(debug)
  fprintf('\n[INFO] movie TStep:%d\n',movieTStep);
end;
tArray = [double(movieStartEpoch); double(movieEndEpoch); movieTStep];

[flag value]    = get_custom_par('CHANNEL',parFile); % channel
[value1 value2] = strread(value,'%s%s','delimiter','_');
technology      = value1{1};
channel         = value2{1};
if(debug)
  fprintf('\n[INFO] channel:<%s>\n',channel);
end;
%
% ==========================================
% CODE: code starts here 
% ==========================================
%
frame_count = 0;
status_fig  = 0;

%
% read shoreline, political boundary and topo data and dot colors for map
%
shore     = importdata(shoreline);
boundary  = importdata(boundaries);
political = importdata(political);

%
% in the custom GMV we expand topo to the world level
% samples are every 20 minutes
%
load(topoFile,'values');

if (minLatIn >= -89)
   minLatIn1 = minLatIn -1;
else
   minLatIn1 = minLatIn;
end; % if

if (maxLatIn <= 89)
   maxLatIn1 = maxLatIn +1;
else
   maxLatIn1 = maxLatIn;
end; % if

if (minLonIn >= -179)
   minLonIn1 = minLonIn -1;
else
   minLonIn1 = minLonIn;
end; % if

if (maxLonIn <= 179)
   maxLonIn1 = maxLonIn +1;
else
   maxLonIn1 = maxLonIn;
end; % if

if(minLonIn1 <0)
   minLonIn2 = minLonIn1 +180; %zero for this topo file is at 180!
else
   minLonIn2 = minLonIn1 +180; %zero for this topo file is at 180!
end;

if(maxLonIn1 <0)
   maxLonIn2 = maxLonIn1 +180; %zero for this topo file is at 180!
else
   maxLonIn2 = maxLonIn1 +180; %zero for this topo file is at 180!
end;

xi=[floor(minLonIn1*60):topoFileInterval:ceil(maxLonIn1*60)];
xi = xi/60.;
xi = xi - centerLon;

xIndex1 = int32(floor(minLonIn2*60/topoFileInterval)+1);
xIndex2 = int32(ceil(maxLonIn2*60/topoFileInterval));

yi=[floor(minLatIn1*60):topoFileInterval:ceil(maxLatIn1*60)];
yi=yi/60;
yi = ((asinh(tan(yi*pi/180.))* 180.0 / pi));

yIndex1 = int32(floor(90+minLatIn1)*60/topoFileInterval+1);
yIndex2 = int32(ceil(90+maxLatIn1)*60/topoFileInterval+1);

if(debug)
   fprintf('\n[INFO] xindex1, xindex2, yindex1, yindex2: %d, %d, %d, %d\n',xIndex1,xIndex2,yIndex1,yIndex2);
end; % if debug

xStep = 1;
yStep = 1;
if(xIndex1 > xIndex2)
   xStep =-1;
end;
if(yIndex1 > yIndex2)
   yStep =-1;
end;

%
% adjustment to make sure vector lengths match and no error due to rounding or step size
%
xIndex2 = xIndex2-xStep*(length(xIndex1:xStep:xIndex2)-length(xi));
yIndex2 = yIndex2-yStep*(length(yIndex1:yStep:yIndex2)-length(yi));

zi= values(yIndex1:yStep:yIndex2,xIndex1:xStep:xIndex2);

load dotcolors % loads array cmap

if(debug)
   fprintf('\n[INFO] Pre-processing complete\n');
end;

%
% ------------------------------------------
% read seismograms, filter and resample
% ------------------------------------------
%
if(debug)
   fprintf('\n[INFO] read waveforms:%s/%s \n',dataPath,strcat('*',channel,'.*',sacFileExtension));
end;

[staName staCoord seisData] = getSeis(ftype, filter, dataPath, tArray, strcat('*',channel,sacFileExtension));
lon = staCoord(:,1);
lat = staCoord(:,2);

%
% ------------------------------------------
% set movie parameters
% ------------------------------------------
%
% get distance to "center" of array
%
viewCount=0;
for i=1:length(lat)
   if(lat(i)>= minLatIn && lat(i) <= maxLatIn && lon(i) >= minLonIn && lon(i) <= maxLonIn)
      viewCount = viewCount+1;
      viewLat(viewCount) = lat(i);
      viewLon(viewCount) = lon(i);
   end;%if
end;%for

if(viewCount==0)
   viewLat = lat;
   viewLon = lon;
end;%if

medianStaLat = median(viewLat);
medianStaLon = median(viewLon);

if(isEventSet)
   if(debug)
      fprintf('[INFO] median(medianStaLat,medianStaLon,eventLat,eventLon) =(%5.3f,%5.3f,%5.3f,%5.3f\n)',medianStaLat,medianStaLon,eventLat,eventLon);
   end;
   [del az] = stadis2(medianStaLat,medianStaLon,eventLat,eventLon);
   if(debug)
      fprintf('[INFO] median(medianStaLat,medianStaLon,del,az) =(%5.3f,%5.3f,%5.3f,%5.3f\n)',medianStaLat,medianStaLon,del,az);
   end;

   %
   % compute some nominal travel times to array center
   %
   p1 = pSlowness * del; 
   r1 = rSlowness * del; 
   r2 = rSlowness * (360-del);

end;

%
% ------------------------------------------
% select and prepare reference seismogram for bottom of plot
% ------------------------------------------
%
% select seismogram in file # refIndex near the center
%
if(debug)
   fprintf('[INFO] %s\n','choosing new reference trace close to the "center" of the array');
end;

refIndex = 0; % index of the reference station
prevDel = 9999;
for i=1:length(lat)
   [delr azr] = stadis2(lat(i),lon(i),medianStaLat,medianStaLon);
    if (delr <= prevDel) && ~isnan(lon(i))
        refIndex    = i;
        prevDel  = delr;
    end
end

if refIndex < 1, return; end; % reference station was not found!!

%
% reference station data with respect to event
%
refLat  = lat(refIndex);
refLon  = lon(refIndex);
refSta  = strtrim(strcat(staName(refIndex,:)));

if(isEventSet)
   [del az] = stadis2(eventLat,eventLon,refLat,refLon);

   refAz   = az;
   refDel  = round(del);

   if(debug)
      fprintf('[INFO] ref sta :%s\n',refSta);
      fprintf('[INFO](reference station & distance:refIndex,refSta,refLat,refLon,round(refDel)): %d %s at %5.3f,%5.3f (%d)',refIndex,refSta,refLat,refLon,round(refDel));
   end;
end;

tStart(refIndex)=movieStartEpoch;
npts(refIndex)=length(seisData(refIndex,:));

is = max( [ceil( int32((movieStartEpoch - tArray(1)) / movieTStep )) + 1 1] );    % starting index of window
movieStartEpoch = tArray(1) + (is -1) * movieTStep;

ie = min( [floor( int32((movieEndEpoch   - movieStartEpoch) / movieTStep )) - 1 npts(refIndex)] ); % ending index of window
movieEndEpoch = movieStartEpoch + (ie -1) * movieTStep;

%
% store the desired portion of data
%
movieData = zeros(length(seisData(:,1)),ie-is+1);

%
% grab the reference station trace before scaling
%
refData = zeros(ie-is+1);
refData = seisData(refIndex,is:ie);

for i=1:length(seisData(:,1))
   movieData(i,:) = seisData(i,is:ie);
end;
movieData = scaleRows(movieData, magnification);%Each station is individually scaled to between [-magnification +magnification]

%
% suggested by Debi Kilb 2011-07-12
%
% 1) Initialize the movieT parameter to be an array and not a matrix by adding in that '1' when you use zeros
% 2) use 'double(p)' so it doesn't force the value to be an integer.
%
movieT = zeros(length(movieData(1,:)),1); % movie trace time since the event time
for p=is:ie
    movieT(p-is+1) = (double(p) - 1)*movieTStep; % time scale for reference seismogram
end;

if(debug)
   fprintf('[INFO] movie start T: %d end T: %d',movieT(1),movieT(length(movieT)));
end;

%
% calculate phases
%
if(isEventSet)
   if(debug)
     fprintf('[INFO] eventDepth=%f,phaseList=%s,[refLat=%f refLon=%f],[eventLat=%f,eventLon=%f]\n',eventDepth,phaseList,refLat,refLon,eventLat,eventLon);
   end;

   phaseTime = taupTime('iasp91',eventDepth,phaseList,'sta',[refLat refLon],'evt',[eventLat,eventLon]);

   if(debug)
      for i=1:length(phaseTime)
         fprintf('[INFO] phase: %s time: %f',phaseTime(i).phaseName, phaseTime(i).time);
      end;
   end;
end;

%
% When we plot the reference time series later on, there are certain
% circumstances, which I cannot entirely identify, which cause the x axis
% line not to plot if the time series points are the same as the Y axis
% limits. So, I enlarge the Y axis limits by 4% and that seems to fix the
% problem. Enlarging the limits by "eps" doesn't do the trick. Ughh!
%
smin = min(movieData(refIndex,:)); % min seismogram amplitude in window used for movie
smax = max(movieData(refIndex,:)); % max seismogram amplitude in window used for movie
smin = smin - 0.01 * abs(smin);
smax = smax + 0.01 * abs(smax);

if(debug)
   fprintf('[INFO] movie Data %d\n',movieData(refIndex,:));
   fprintf('[INFO] processed reference trace\n');
end;

headline=plotTitle;

%
% ------------------------------------------
% ADJUST COORDINATES TO MERCATOR PROJECTION (centered on the array "center" longitude)
%
%
% process data points for scatter plopt
%
% ------------------------------------------

%
% stations
%
if(debug)
  fprintf('\n[INFO] lon - centerLon:%d\n',size(centerLon));
end;

lon = lon - centerLon;
lat = asinh(tan(lat*pi/180.))* 180.0 / pi;

%
% boundaries
%
shore.data(:,1) = shore.data(:,1) - centerLon;
shore.data(:,2) = asinh(tan(shore.data(:,2)*pi/180.))* 180.0 / pi;
boundary.data(:,1) = boundary.data(:,1) - centerLon;
boundary.data(:,2) = asinh(tan(boundary.data(:,2)*pi/180.))* 180.0 / pi;
political.data(:,1) = political.data(:,1) - centerLon;
political.data(:,2) = asinh(tan(political.data(:,2)*pi/180.)) * 180.0 / pi;

%
% ------------------------------------------
% Open figure and plot static part of movie
% ------------------------------------------
%
fig_h  = figure(); %turns visibility of figure off
set (fig_h,'Position',[1 150 screenWidth screenHeight],'Color',[1 1 1],'Units','pixels');

set (fig_h,'RendererMode','manual','Renderer','zbuffer');

% create the base map
subplot(4,1,1:3);
set(gca,'Color',[1 1 1 ]);
hold on

if(debug)
   fprintf('\n[INFO] x,y,Z,%d, %d, %d\n',length(xi),length(yi),size(zi));
   fprintf('\n[INFO] contourf [xi],[yi],[zi]: [%d, %d], [%d, %d], [%d %d]\n',xi(1),xi(length(xi)), yi(1),yi(length(yi)),zi(1),zi(size(zi)));
end; % if debug
contourf(xi,yi,zi,[-10000 0],'EdgeColor','blue'); % shade the continent with a nice grey color
colormap(cmap2);

plot(political.data(:,1),political.data(:,2),'k');
plot(boundary.data(:,1),boundary.data(:,2),'k');


%
% % the fill above masks the Great Lakes, now put the Great Lakes back with blue color
% %
% load conus;
% gtlon=gtlakelon-centerLon;
% gtlat= asinh(tan(gtlakelat*pi/180.))* 180.0 / pi;
% 
% % lakes are separated by NaN
% % Use polysplit to create two cell arrays, latc and lonc
% %
% [latc,lonc]=polysplit(gtlat,gtlon);
% fill(lonc{1},latc{1},'w');
% fill(lonc{2},latc{2},'w');
% fill(lonc{3},latc{3},'w');

%
% limits for projected coordinates
%
xlim([minLon maxLon]);
if(debug)
   fprintf('[INFO] min & max lon(%d %d)\n',minLon,maxLon);
end;

ylim([minLat maxLat]);
if(debug)
   fprintf('[INFO] min & max lat(%d %d)\n',minLat,maxLat);
end;

set(gca,'Box','on','XTick',[],'YTick',[]);
title(headline,'FontSize',headerFontSize,'FontWeight','bold');

if(debug)
   fprintf('[INFO] visualization x limits: %5.3f , %5.3f\n',xlim);
   fprintf('[INFO] visualization y limits: %5.3f , %5.3f\n',ylim);
end;

%
% calculate the great circle path
%
if(isEventSet)
   %distgc = distance(eventLat,eventLon,refLat,refLon);
   distgc = sqrt((eventLat-refLat)^2 + (eventLon-refLon)^2);
   azgc = azimuth(eventLat,eventLon,refLat,refLon);

   if(debug)
      fprintf('[INFO] (distgc, azgc): (%5.3f , %5.3f)\n',distgc, azgc);
   end;

   [latgc,longc] = track1('gc',eventLat,eventLon,azgc,distgc);

   if(debug)
      fprintf('[INFO] XYGC count: %d\n',length(latgc));
      for i=1:length(latgc)
         fprintf('[INFO] latGC: %5.3f lonGC: %5.3f\n',latgc(i),longc(i));
      end;
   end;

   %
   % here we extract the GC path that falls within the view port
   %
   visibleLonGc = [];
   visibleLatGc = [];
   visibleCount=0;
   for i=1:length(latgc)
      longc(i) = longc(i) - centerLon;
      latgc(i) = asinh(tan(latgc(i)*pi/180.))* 180.0 / pi;
      if((latgc(i) >= minViewLat && latgc(i) <= maxViewLat) && (longc(i) >=minViewLon && longc(i) <= maxViewLon))
         visibleCount = visibleCount +1;
         visibleLonGc(visibleCount) = longc(i);
         visibleLatGc(visibleCount) = latgc(i);
         if(debug)
            fprintf('[INFO] visibleLatGc: %5.3f visbleLonGc: %5.3f\n',visibleLatGc(visibleCount),visibleLonGc(visibleCount));
         end;
      end;
   end;
end;


%
% ------------------------------------------
% plot the seismogram
% ------------------------------------------
%
subplot(4,1,4:4);
hold on
sampStart = 1;

plot ( movieT,movieData(refIndex,:),'LineWidth',1, 'LineSmoothing','on','color','k' );
text(movieT(5),0.1,strcat(refSta,'/',channel),'FontSize',10,'VerticalAlignment','bottom');

if(debug)
   fprintf('[INFO] trace smin , smax %d , %d\n',smin,smax);
end; % if debug

plot ( [movieT(1) movieT(length(movieT))], [smin smin], 'k' );
xlim(gca, [movieT(1) movieT(length(movieT))]);
ylim(gca, [smin smax*1.25]);

%
% trace label
%
tLabel = ' ';
if(isEventSet)
   tLabel = sprintf('%s%5.1f%1s/%d km  Azimuth %5.1f%s%5.1f%1s%5.1f','  Distance ',refDel,'\circ',round(deg2km(refDel)),refAz,'\circ' );
end;

if(debug)
   fprintf('[INFO] trace label tmin, smax, tLabel %d , %d, %s\n',movieT(1),smax,tLabel);
end;

%text(double(movieT(1)+(movieT(length(movieT))-movieT(1))/2.3),double(1.36*smax),tLabel,'FontSize',10,'VerticalAlignment','bottom');

if(debug)
   fprintf('[INFO] trace tmin, tmax %d , %d, %s\n',movieT(1),movieT(length(movieT)));
end;

%
% time axis label
%
xlabel('  Visualization Elapsed Time (hh:mm:ss)  ','FontSize',12,'FontWeight','bold');


xs = movieT(length(3));
ys = 1.6*smax;
text(xs,ys,['\fontsize{9}vertical motion   {\fontsize{6}\color{red}\otimes\fontsize{9} up   \fontsize{6}\color{blue}\otimes\fontsize{9} down}'])

%
% do not plot y-axis
%
set(gca,'YTick',[],'YColor','w');

%
% to have regular tick labels at every xTick seconds
%
tmin = ceil(1.0*movieT(1)/xTick)*xTick;
tmax = floor(1.0*movieT(length(movieT))/xTick)*xTick;
mCount = 1 + (tmax - tmin)/xTick;

if(debug)
   fprintf('[INFO] tmin tmax mCount %d , %d, %d\n',tmin,tmax,mCount);
end;

ticks = zeros(mCount);
tickValue = tmin;
for i=1:mCount
   ticks(i) = tickValue;
   tickValue = tickValue + xTick;
   tickCount = i;
end;

%
% relabel the time ticks
%
%xTicks = get(gca,'XTick'); % get the x values for each tick mark
xTicks = ticks;

if(debug)
   fprintf('[INFO] time axis tick values: %d\n',xTicks);
end;

xTickLabels = {};
doSkip = 0; % we want to plot every other label
for i=1:length(xTicks)
   if(doSkip <=0)
      hLabel = idivide(xTicks(i),int32(3600));
      mLabel = idivide(mod(xTicks(i),3600),int32(60));
      sLabel = xTicks(i)-hLabel*3600-mLabel*60;
      xTickLabels{i}  = sprintf('%02d:%02d:%02d',hLabel,mLabel,sLabel);

      if(debug)
         fprintf('[INFO] time axis tick labels: %s\n',xTickLabels{i});
      end;

      doSkip = 1;
   else
      xTickLabels{i} = ' ';
      doSkip = 0;
   end;
end;

%
%set the time axis (no limit change, just the labels)
%
set(gca,'XTick', [ticks(1):xTick:ticks(tickCount)]); % tick locations
set(gca,'XTickLabel', xTickLabels); % label each tick with a string

%
% ------------------------------------------
% now, plot the phases
% ------------------------------------------
%
oldPhaseTime=-999;
shift=1;

if(isEventSet)
   for i=1:length(phaseTime)
      phaseEpoch = eventEpoch+phaseTime(i).time;
      if(phaseEpoch < movieStartEpoch || phaseEpoch > movieEndEpoch)
         continue;
      else
         phaseT = double(phaseEpoch - movieStartEpoch);
      end;
      if(debug)
         fprintf('[INFO] phase time #%d: %d %s\n',i,phaseTime(i).time,phaseTime(i).phaseName);
         fprintf('[INFO] phase oldtime time spacing #%d: %d %d %d\n',i,oldPhaseTime,phaseSpacing);
      end; % if debug
      if(i > 1)
         if(~strcmp(phaseTime(i).phaseName, phaseTime(i-1).phaseName) && (phaseT-oldPhaseTime) > phaseSpacing)
            plot ([phaseT,phaseT],[smin,smax*0.8],'blue'); % the blue phase line
            text(phaseT, smax*shift, phaseTime(i).phaseName, 'Color', 'blue','fontsize',7);
            oldPhaseTime = phaseT;

            % shift phase vertical alignment for a better readbility
            if(shift == 1)
               shift=1.1;
            else
               shift=1;
            end
         end;
      else
         plot ([phaseT,phaseT],[smin,smax*0.8],'blue'); % the blue phase line
         text(phaseT, smax*shift, phaseTime(i).phaseName, 'Color', 'blue','fontsize',7);
         oldPhaseTime = phaseT;
         % shift phase vertical alignment for a better readbility
         if(shift == 1)
            shift=1.1;
         else
            shift=1;
         end
      end;
   end;
end

%
% R1 & R2 & I
%
if(isEventSet)
   shift=1.1;
   r1Time = (distgc * rSlowness);
   r1Epoch = eventEpoch+r1Time;
   if(debug)
      fprintf('[INFO] r1 phase time #%d %d %d\n',r1Time,r1Epoch,eventEpoch);
   end; % if debug
   if(r1Epoch >= movieStartEpoch && r1Epoch <= movieEndEpoch) 
      r1T = double(r1Epoch - movieStartEpoch);
      if(debug)
         fprintf('[INFO] r1Epoch movieStartEpoch r1T %d %d %d\n',r1Epoch,movieStartEpoch,r1T);
      end; % if debug
      if(del > 20)
           plot ([r1T,r1T],[smin,smax*0.8],'blue'); % the blue phase line
           text(r1T, smax*shift, 'R1', 'Color', 'blue','fontsize',7);
      end;
   end

   r2Time = ((360 - distgc) * rSlowness);
   r2Epoch = eventEpoch+r2Time;
   if(r2Epoch >= movieStartEpoch && r2Epoch <= movieEndEpoch) 
      r2T = double(r2Epoch - movieStartEpoch);
      if(del > 20)
           plot ([r2T,r2T],[smin,smax*0.8],'blue'); % the blue phase line
           text(r2T, smax*shift, 'R2', 'Color', 'blue','fontsize',7);
      end;
   end;

   if(strcmp(technology,'INFRA'))
      iTime = distgc * iSlowness;
      if( iTime < tmax)
           plot ([iTime,iTime],[smin,smax*0.8],'blue'); % the blue phase line
           text(r2Time, smax*shift, 'I', 'Color', 'blue','fontsize',7);
      end;
   end;

end;
%
% set up output AVI file name
%
[iy,id,ih,im,ss] = ep2dat(eventTime);

mname = sprintf('%s/%s_%s',moviePath,moviePrefix,movieId);

if(debug)
   fprintf('[INFO] visualization file: %s\n',mname);
end;

%
% Web link file
%

[iy,id,ih,im,ss] = ep2dat(movieStartEpoch);
dateNum = (datenum([1970 1 1 0 0 0])*86400 + double(movieStartEpoch)) / 86400;
eqDateTime =datestr((datenum([1970 1 1 0 0 0])*86400 + double(eventTime)) / 86400, 'yyyy/mm/dd HH:MM:SS');
startDateTime =datestr((datenum([1970 1 1 0 0 0])*86400 + double(movieStartEpoch)) / 86400, 'yyyy/mm/dd HH:MM:SS');
endDateTime =datestr((datenum([1970 1 1 0 0 0])*86400 + double(movieEndEpoch)) / 86400, 'yyyy/mm/dd HH:MM:SS');

%
% here we want to create an AVI movie without displaying the figure!
%
aviobj = VideoWriter(mname, 'Uncompressed AVI'); % create AVI file
aviobj.FrameRate = 25;
open(aviobj);

% Main loop over frames of movie, extracting time time slice from network
% on each loop
%
frame = 0;
frameTotal = 0;
frameTrip = 0;
t = movieStartEpoch - movieTStep;

%
% one movie frame at a time
%
fprintf('\n[INFO]FRAME 0');


for tStep=1:length(movieT)
    t = movieStartEpoch + (tStep-1) * movieTStep;
    frame = frame + 1;
    
    %
    % do all stations for this time slot
    %
    for staIndex=1:length(movieData(:,tStep))

        %
        % Select color for this station-amplitude value.
        % Colormap has 64 values going from negative to positive, so map amplitude to a 1-to-64 scale
        % We assume peak-to-peak amplitudes in seismograms are +-1 - so amplitudes outside this range
        % will saturate color scale.
        % If amp is outside of +-1 then color scale saturates (we may have
        % scaled seismogram to values larger than +-1, so that small
        % amplitudes will be visibile.
        % 
        %
        % manoch - 2010-11-26 added check on NaN to avoid plotting blue dots when we only have waveform segments
        % 
        if isnan(movieData(staIndex,tStep))
           z = 32;
        else
           z = int32(32.5 * movieData(staIndex,tStep) + 31.5);
        end;

        if(debug)
           fprintf('[INFO] Z:%d\n',z);
        end;

        if z<1
            z=1;
        elseif z>64
            z=64;
        end;
        col(staIndex,1:3) = cmap(z,1:3);

        if(debug)
           fprintf('[INFO] Color:%d %d %d\n',col(staIndex,1:3));
        end;
    end;
    %
    if frameTrip > 0
        delete(h1);
        delete(h2);
        delete(h3);
        delete(h4);
    end;
    frameTrip = frameTrip + 1;

    %
    % plot the station dots
    %
    fprintf('.');
    subplot(4,1,1:3);
    h1=scatter(lon,lat,markerSize,col,'filled','MarkerEdgeColor',markerEdgeColor);
    h2=scatter(lon(refIndex),lat(refIndex),refMarkerSize,circleColor); % yellow circle around ref station
    set(h2,'LineWidth',circleThickness); % make circle even thicker!

    %
    % calculate the great circle path (we do it for each frame to make sure that it is not covered by other points
    %
    % plot the event location. Because of the view port, this will only be visible for the US events
    %
if(isEventSet)
    plot(eventLon - centerLon,asinh(tan(eventLat*pi/180.))* 180.0 / pi,'rp',...
       'MarkerSize',30,'MarkerFaceColor','r');
end;
  
if(showGCPath)
    plot(visibleLonGc,visibleLatGc,'y-','LineWidth',1);
 
end;
    %arrow([longc(end-3),latgc(end-3)],[longc(end),latgc(end)],'y');

    %
    % plot the time stamp
    [iy,id,ih,im,ss] = ep2dat(t);
    %
    % change the time stamp from Julian day to regular date
    % 
    dateNum = (datenum([1970 1 1 0 0 0])*86400 + double(t)) / 86400;
    traceTitleString = sprintf('%s%s%s%s  %s%s%s',datestr(dateNum, 'yyyy/mm/dd HH:MM:SS'),' UTC (',num2str(round(movieT(tStep))),' s)');
    h3=text(minLon+timeHorizontalOffset,minLat+timeVerticalOffset,[traceTitleString,filterString,dtString],'FontSize',12); 

    %
    % now, plot the moving red line on the seismogram
    subplot(4,1,4:4);
    h4=plot ([movieT(tStep),movieT(tStep)],[smin,smax],'red'); % the red moving line

    %
    % IRIS logo
    %
    has = axes('Position',irisLogoPosition);
    I=imread(irisLogo);
    image(I);
    daspect([1 1 1]);
    axis off;

    %
    % grab the frame, increment the time step, and loop around
    %
    pause(0.01);
    F(frame) = getframe(gcf);
    if frame > frameFlushInterval
        fprintf(' >>>>>');
        if(debug)
           fprintf('[INFO] flushing frames\n');
        end;
        %aviobj = addframe(aviobj,F);
        writeVideo(aviobj,F);
        frameTotal = frameTotal + frame -1;

        if(debug)
           fprintf('[INFO] frames total: %d\n',frameTotal-1);
        end;

        fprintf('\n[INFO]FRAME %d ',frameTotal);
        frame = 0;
    end
    t = t + movieTStep;
end;

if frame > 0
    frameTotal = frameTotal + frame -1;
    fprintf(' >>>>>');
    if(debug)
       fprintf('[INFO] flushing the %d remaining frames\n',frame);
    end;
    %aviobj = addframe(aviobj,F(1:frame));
    writeVideo(aviobj,F(1:frame));
        
    if(debug)
       fprintf('[INFO] frames total: %d\n',frameTotal-1);
    end;
end;

pause(1);
status_fig = close(gcf);
pause(5);
if(debug)
   fprintf('[INFO] total frames: %d\n',frameTotal);
end;
close(aviobj); % closes the AVI file
clear F sacdat sachdr xi yi zi lat lon shore boundary political npts;
pause(0.5);
