function [frame_count,status_fig] = gmv_cust_1_dpl(dataPath,ename,elabel)
%
% This is the "production" version of the code that is used to produce the
% movies that go on the USArray web site.
%
% History:
%
% 2009 Bob Woodward (woodward@iris.edu): Original version 
% 	This code is far from a hardened, truly "production" code, but it is what
% 	it is!
%
% 	The versions of the code leading up to this one are named
% 	TA_animation_chuck_sod.
%
% 	Inspired by, and modeled after, the work of Chuck Ammon, Pennsylvania
% 	State University.
%
% 	Assumes data are in SAC files that are in MATLAB path.
% 	works for BH or LH data - or other sample rates
% 	should work even if sample rates are mixed
%
% 	lots could be done to clean this up, such as:
%	  - pre-allocate memory for collecting the frames (if possible)
%	  - tidy up the FORTRAN-like code using better matlab code
%	  - eliminate calls to "epoch" and "ep2dat" - I think these can both be
%	    done cleanly with MATLAB built-ins
%	  - modularize!
%	  - improve the event selection strategy. For example, if I wrote the
%	    event info into SEED files when I do breq_fast requests then this
%	    informatio would already be in SAC headers.
%
% 2009-11-16 Manochehr Bahavar (manoch@iris.washington.edu): move to IRIS DMC
% 2009-11-23 Manoch: reorganized
% 2009-12-22 Manoch: changed the time stamp format from Julian day to regular date format
%                    changed the time axis from seconds to hh:mm:ss
% 2009-12-23 Manoch: to avoid tick label overlaps, now plots every other time tick label
% 2010-01-04 Manoch: the following enhancements are made:
%                    - reference stations is ate the "center" of the array
%                    - display window is relative to the reference station based on the given parameters
%                    - moved the time stamp text below the plot window
%                    - filter parameters are displayed (BW)   
% 2010-01-05 Manoch: expanded the sample trace's l;abel to include, channel, azimuth, distance in km, ...
% 2010-01-15 Manoch: many changes to the format and layout. Now adds links for map and waveforms
% 2010-01-22 Manoch: many changes to the format and layout. Now accepts elabel (event label) for display on web
% 2010-02-02 Manoch: final release version. Passing more info to the supplement page
% 2010-02-08 Manoch: R1.0 installed
% 2010-04-01 Manoch: added map file output
% 2010-04-30 Manoch: now displays whole US, seconds and time for the seismogram and also shows selected phases
% 2010-05-05 Manoch: added the great circle path
% 2010-05-06 Manoch: added the R1&R2 travel times
% 2010-06-30 Manoch: R1.5 This is a working version with the following changes:
%                    Viewport has been extended to cover the entire US
%                    Phases are marked on the trace
%                    titles are reorganized
% 2010-07-01 Manoch: R2.0 In this version I utilized Lloyd' scheme to read traces and join individual segments
%                    which is a more efficient way of handling traces. This opens doors to implementing the
%                    3D version.
% 2010-07-28 Manoch: R2.0 preparing for test. Changes Tstep to 4( was 5) for smoother video
% 2010-11-26 Manoch: Added check on NaN to avoid plotting blue dots when we only have waveform segments
% 2011-09-22 Manoch: migrated to dplive
% 2012-01-09 Manoch: added Debi Kilb's code to list the filter and time step
% 2012-01-10 Manoch: made the magnification parameter user selectable
% 2012-03-26 Manoch: Updates due to MATLAB R2012a upgrade:
%                        - Undefined function 'plus' for input arguments of type 'struct'
%                          Now have to explicitly spicy ".data" when using shore, boundary and topo structures 
% 2012-04-11 manoch: moved the TMP directory location via setenv('TMP','/products/scratch/product2/TMP');
%
%
% ==========================================
% CONFIGURATION: configuration parameters
% ==========================================
%
setenv('TMP','/products/scratch/product2/TMP');

%
% needed files
%
shoreline             = 'world_coast.dat';
boundaries            = 'world_boundaries.dat';
political             = 'US_political.dat';
%topoFileInterval      = 20; % 20 seconds
%topoFile              = 'topolores.mat';
topoFileInterval      = 5; % 5 seconds
topoFile              = 'topohires.mat';
eventFile             = 'event.asc';
parFile               = strcat(dataPath,'/',eventFile);
moviePath             = '/products/usarraygmv/production/TSD_dpl/gmv_cust_movie';
irisLogo              = '/products/usarraygmv/production/matlab/image/IRIS_Logo_Url.png';

%
% constants to have
%

irisLogoRight         = 0.0;
irisLogoUp            = 0.005;
irisLogoWidth         = 0.45;
irisLogoHeight        = 0.050;
irisLogoPosition      = [irisLogoRight,irisLogoUp,irisLogoWidth,irisLogoHeight]; %Lowerleft,size
irisUrlPosition       = [irisLogoRight+irisLogoWidth*1.2,irisLogoUp*1.2,irisLogoWidth,irisLogoHeight]; %Lowerleft,size

pSlowness             = 7.2;    % P slowness (s/degree);  a P slowness of ~ 7.2 s/deg (should be conservative out to 125 deg)
rSlowness             = 28.5; % Rayleigh slowness (s/degree); originally 29.835 but using slowness of 28.5 to provides a better phase display
%                       infrasound celerities often used: 330 m/s, 295 m/s 303 m/s 
iSlowness             = 111.195 / 0.29; % Infrasound slowness (s/degree)
magnification         = 10; %Each station is individually scaled to between [-magnification and +magnification]
phaseList             = 'P,Pn,Pg,Pdiff,pPdiff,sPdiff,PcP,PP,PP,PS,PKP,SKS,S,Sn,Sg,Sdiff,pSdiff,sSdiff,SP,ScS,SS,SKS'; %list of seismic phases to calculate travel times for
phaseSpacing          = 200; % number of seconds between two consecutive phases to avoid over plots
eventDepth            = 0;
screenWidth           = 800;
screenHeight          = 600;
aspectRatio           = screenWidth/screenHeight;
frameFlushInterval    = 50;
leftMargin            = 12; % longitude margin to the west
rightMargin           = 16; % longitude margin to the west

headerFontSize        = 14;
circleThickness       = 3;  % thickness of the circle around the reference station
circleColor           = 'yellow'; % color of the circle around the reference station
markerEdgeColor       = 'k'; %Marker edge color. The color of the marker or the edge color for filled markers (circle, square, diamond, pentagram, hexagram, and the four triangles). ColorSpec  defines the color to use. none specifies no color, which makes nonfilled markers invisible. auto sets MarkerEdgeColor  to the same color as the CData property.
markerSize            = 55; % circle marker size for the stations
refMarkerSize         = 180; % circle marker size for the reference station

xTick                 = 86400; % seismogram tick spacing in seconds
smapWidth             = 700;
smapHeight            = 600;

% cmap2                 = [1.0 1.0 1.0; 0.9 0.9 0.9]; % alternate color map for shading the US for 'US_topodata_fine.mat'
cmap2                 = [1 1 1; 1 1 1; 1 1 1; 0.9 0.9 0.9];% alternate color map for shading the US for 'topolores.mat'
ftype                 = 'NONE';
filter                = [0 0];
debug                 = false;
%
% set parameters
%
[flag value]          = get_custom_par('TITLE',parFile); 
if (flag > 0)
  plotTitle = value;
  if(debug)
    fprintf('\n[INFO] plotTitle:%s\n',plotTitle);
  end;
end;

filterString = ['; no filter applied'];
[flag value]          = get_custom_par('HP',parFile);
hp = 0;
if (flag > 0)
  flow = str2double(value);
  filter = [flow;flow];
  hp = 1;
  ftype = 'HP';
  filterString = [' Filter (Hz): ',num2str(flow),' highpass'];
  if(debug)
    fprintf('\n[INFO] %s filter frequency:%d\n',ftype,flow);
  end;
end;

[flag value]          = get_custom_par('LP',parFile);
lp=0;
if (flag > 0)
  fhigh = str2double(value);
  filter = [fhigh;fhigh];
  lp = 1;
  filterString = [' Filter (Hz): ',num2str(fhigh),' lowpass'];
  ftype = 'LP';
  if(debug)
    fprintf('\n[INFO] %s filter frequency:%d\n',ftype,fhigh);
  end;
end;

if(hp==1 && lp==1)
   ftype = 'BP';
   filter = [flow;fhigh];
   filterString = [' Filter (Hz): ',num2str(flow),' to ',num2str(fhigh)];
  if(debug)
    fprintf('\n[INFO] %s filter frequency:%d-%d\n',ftype,flow,fhigh);
  end;
end;
 

isEventSet = false;
[flag value]          = get_custom_par('EVENT_LAT',parFile); 
if (flag >0 )
  eventLat = str2double(value);
  isEventSet = true;
  [flag value]          = get_custom_par('EVENT_LON',parFile);
  if (flag > 0)
    eventLon = str2double(value);
  end;

  if(debug)
    fprintf('\n[INFO] event lat/lon:%f to %f\n',eventLat,eventLon);
  end;
  [flag value]     = get_custom_par('EVENT_TIME',parFile);

  eventTime  = value;

  if(debug)
    fprintf('\n[INFO] event:<%s>\n',eventTime);
  end;

  [value1 value2] = strread(eventTime,'%s%s','delimiter',' ');
  eventDate  = value1{1};
  eventTime  = value2{1};
  if(debug)
    fprintf('\n[INFO] event start:%s T %s\n',eventDate,eventTime);
  end;

  [value1 value2 value3] = strread(eventDate,'%s%s%s','delimiter','-');
  eventYear  = str2double(value1{1});
  eventMonth = str2double(value2{1});
  eventDay   = str2double(value3{1});
  if(debug)
    fprintf('\n[INFO] event Date:%d/%d/%d\n',eventYear,eventMonth,eventDay);
  end;

  [value1 value2 value3] = strread(eventTime,'%s%s%s','delimiter',':');
  eventHour  = str2double(value1{1});
  eventMinute = str2double(value2{1});
  eventSecond = str2double(value3{1});
  if(debug)
    fprintf('\n[INFO] event Time:%d-%d-%d\n',eventHour,eventMinute,eventSecond);
  end;

  eventJday        = jday(eventYear, eventMonth, eventDay); % get julian day
  eventEpoch       = epoch(eventYear,eventJday,eventHour,eventMinute,eventSecond); %now, translate all events times into epochal
  if(debug)
    fprintf('\n[INFO] event Start Jday,epoch:%d,%d\n',eventJday,eventEpoch);
  end;
end;


[flag value]          = get_custom_par('LL_LAT',parFile); % map extent to the south
if (flag > 0)
  minLatIn = str2double(value);
  minLat = ((asinh(tan(minLatIn*pi/180.))* 180.0 / pi));
end;

[flag value]          = get_custom_par('UR_LAT',parFile); % map extent to the north
if (flag > 0)
  maxLatIn = str2double(value);
  maxLat = ((asinh(tan(maxLatIn*pi/180.))* 180.0 / pi));
end;

if(debug)
  fprintf('\n[INFO] min,max lat:%d,%d\n',minLat,maxLat);
end;

[flag value]          = get_custom_par('LL_LON',parFile); % map extent to the west
if (flag > 0)
  minLonIn = str2double(value);
  minLon = minLonIn;
end;

[flag value]          = get_custom_par('UR_LON',parFile); % map extent to the east
if (flag > 0)
  maxLonIn = str2double(value);
  maxLon = maxLonIn;
end;

centerLon             = (minLon+maxLon)/2.0; % set to -999 to center on the array
minLon = minLon - centerLon;
maxLon = maxLon - centerLon;

[flag value]          = get_custom_par('WIDTH',parFile); % map width
if (flag > 0)
  screenWidth = floor(str2double(value));
  if(mod(screenWidth,2) == 1) % make sure width is even so ffmpeg has no problem!
     screenWidth = screenWidth + 1;
   end;
end;

[flag value]          = get_custom_par('MAGNIFICATION',parFile); % trace magnification
if (flag > 0)
  magnification = floor(str2double(value));
end;

if(debug)
  fprintf('\n[INFO] min,max,center lon:%d,%d,%d\n',minLon,maxLon,centerLon);
end;

minViewLat            = minLat; % minimum latitude of the view port
maxViewLat            = maxLat; % maximum latitude of the view port
dlat = abs(maxViewLat - minViewLat);

minViewLon            = minLon; % minimum longitude of the view port
maxViewLon            = maxLon; % maximum longitude of the view port
dlon = abs(maxViewLon - minViewLon);
timeHorizontalOffset  = 0;  % degrees longitude offset of the time stamp from the lower left corner of the map


fprintf('\n[INFO] dlat, dlon:%d,%d\n',dlat,dlon);
%
% adjust video dimensions based on the selected view port
%
screenHeight = floor(screenWidth * (dlat/dlon) * 4.0/3.0);
if(mod(screenHeight,2) == 1) % make sure width is height so ffmpeg has no problem!
   screenHeight = screenHeight + 1;
end;
if (screenHeight > 800)
   screenWidth = ceil(screenWidth * 800/screenHeight);
   screenHeight = 800;
end;
if(mod(screenWidth,2) == 1)%% make sure width is even so ffmpeg has no problem!
   screenWidth = screenWidth + 1;
end;

timeVerticalOffset    = -600*dlat/screenHeight/32.0; % degrees latitude offset of the time stamp from the lower left corner of the map

fprintf('\n[INFO] screen size:%d,%d\n',screenWidth,screenHeight);
   
showGCPath            = false;
[flag value]     = get_custom_par('SHOW_GC',parFile); 
if(flag > 0 && str2double(value) == 1)
   showGCPath=true;
end;

showCC                = false;      % show concentric circles around event
[flag value]     = get_custom_par('SHOW_CC',parFile); 
if(flag > 0 && str2double(value) == 1)
   showCC=true;
end;

[flag movieStart]     = get_custom_par('WIN_START',parFile); % movie start time
if(~isEventSet)
   eventTime = movieStart;
end;

[flag movieEnd]       = get_custom_par('WIN_END',parFile); % movie start time
[flag value]     = get_custom_par('WIN_STEP',parFile); % movie time step, in s

movieTStep  = str2double(value);
dtString = [' Time Step: ',num2str(movieTStep),'s '];

if(debug)
  fprintf('\n[INFO] movie:<%s> to <%s>\n',movieStart,movieEnd);
end;

[value1 value2] = strread(movieStart,'%s%s','delimiter',' ');
movieStartDate  = value1{1};
movieStartTime  = value2{1};
if(debug)
  fprintf('\n[INFO] movie start:%s T %s\n',movieStartDate,movieStartTime);
end;

[value1 value2 value3] = strread(movieStartDate,'%s%s%s','delimiter','-');
movieStartYear  = str2double(value1{1});
movieStartMonth = str2double(value2{1});
movieStartDay   = str2double(value3{1}); 
if(debug)
  fprintf('\n[INFO] movie Date:%d/%d/%d\n',movieStartYear,movieStartMonth,movieStartDay);
end;

[value1 value2 value3] = strread(movieStartTime,'%s%s%s','delimiter',':');
movieStartHour  = str2double(value1{1});
movieStartMinute = str2double(value2{1});
movieStartSecond = str2double(value3{1}); 
if(debug)
  fprintf('\n[INFO] movie Time:%d-%d-%d\n',movieStartHour,movieStartMinute,movieStartSecond);
end;

movieStartJday        = jday(movieStartYear, movieStartMonth, movieStartDay); % get julian day
movieStartEpoch       = epoch(movieStartYear,movieStartJday,movieStartHour,movieStartMinute,movieStartSecond); %now, translate all events times into epochal
if(debug)
  fprintf('\n[INFO] movie Start Jday,epoch:%d,%d\n',movieStartJday,movieStartEpoch);
end;

[value1 value2] = strread(movieEnd,'%s%s','delimiter',' ');
movieEndDate  = value1{1};
movieEndTime  = value2{1};
if(debug)
  fprintf('\n[INFO] movie start:%s T %s\n',movieEndDate,movieEndTime);
end;

[value1 value2 value3] = strread(movieEndDate,'%s%s%s','delimiter','-');
movieEndYear  = str2double(value1{1});
movieEndMonth = str2double(value2{1});
movieEndDay   = str2double(value3{1});
if(debug)
  fprintf('\n[INFO] movie Date:%d/%d/%d\n',movieEndYear,movieEndMonth,movieEndDay);
end;

[value1 value2 value3] = strread(movieEndTime,'%s%s%s','delimiter',':');
movieEndHour  = str2double(value1{1});
movieEndMinute = str2double(value2{1});
movieEndSecond = str2double(value3{1});
if(debug)
  fprintf('\n[INFO] movie Time:%d-%d-%d\n',movieEndHour,movieEndMinute,movieEndSecond);
end;

movieEndJday        = jday(movieEndYear, movieEndMonth, movieEndDay); % get julian day
movieEndEpoch       = epoch(movieEndYear,movieEndJday,movieEndHour,movieEndMinute,movieEndSecond); %now, translate all events times into epochal
if(debug)
  fprintf('\n[INFO] movie End Jday,epoch:%d,%d\n',movieEndJday,movieEndEpoch);
end;

%
% calculate movie length and time label spacing
%
movieLength = movieEndEpoch - movieStartEpoch;
if(movieLength < 2.2*xTick)
   xTick = 86400;
end;
if(movieLength < 2.2*xTick)
   xTick = 44800;
end;
if(movieLength < 2.2*xTick)
   xTick = 22400;
end;
if(movieLength < 2.2*xTick)
   xTick = 11200;
end;
if(movieLength < 2.2*xTick)
   xTick = 9600;
end;
if(movieLength < 2.2*xTick)
   xTick = 4800;
end;
if(movieLength < 2.2*xTick)
   xTick = 2400;
end;
if(movieLength < 2.2*xTick)
   xTick = 1200;
end;
if(movieLength < 2.2*xTick)
   xTick = 600;
end;
if(movieLength < 2.2*xTick)
   xTick = 300;
end;
if(movieLength < 2.2*xTick)
   xTick = 180;
end;
if(movieLength < 2.2*xTick)
   xTick = 120;
end;
if(movieLength < 2.2*xTick)
   xTick = 60;
end;

if(debug)
  fprintf('\n[INFO] movie TStep:%d\n',movieTStep);
end;
tArray = [double(movieStartEpoch); double(movieEndEpoch); movieTStep];

[flag value]    = get_custom_par('CHANNEL',parFile); % channel
[value1 value2] = strread(value,'%s%s','delimiter','_');
technology      = value1{1};
channel         = value2{1};
if(debug)
  fprintf('\n[INFO] channel:<%s>\n',channel);
end;
%
% ==========================================
% CODE: code starts here 
% ==========================================
%
frame_count = 0;
status_fig  = 0;

%
% read shoreline, political boundary and topo data and dot colors for map
%
shore     = importdata(shoreline);
boundary  = importdata(boundaries);
political = importdata(political);

%
% in the custom GMV we expand topo to the world level
%
%xVal =[0:359];
%xi = wrapTo180(xVal);
%yi=[-89:90];
%
%load(topoFile,'topo','topolegend');
%zVal = topo;
%zi=zVal/1000.0;
%load(topoFile,'xi','yi','zi');
%
% minLonIn1, maxLonIn1, minLatIn1, maxLatIn1 have a small pading to make sure that
% it covers the whole view port
%
% samples are every 20 minutes
%
load(topoFile,'values');

if (minLatIn >= -89)
   minLatIn1 = minLatIn -1;
else
   minLatIn1 = minLatIn;
end; % if

if (maxLatIn <= 89)
   maxLatIn1 = maxLatIn +1;
else
   maxLatIn1 = maxLatIn;
end; % if

if (minLonIn >= -179)
   minLonIn1 = minLonIn -1;
else
   minLonIn1 = minLonIn;
end; % if

if (maxLonIn <= 179)
   maxLonIn1 = maxLonIn +1;
else
   maxLonIn1 = maxLonIn;
end; % if

minLonIn2 = minLonIn1;
if(minLonIn1 <0)
   minLonIn2 = minLonIn1 +180; %zero for this topo file is at 180!
else
   minLonIn2 = minLonIn1 +180; %zero for this topo file is at 180!
end;

maxLonIn2 = maxLonIn1;
if(maxLonIn1 <0)
   maxLonIn2 = maxLonIn1 +180; %zero for this topo file is at 180!
else
   maxLonIn2 = maxLonIn1 +180; %zero for this topo file is at 180!
end;

xi=[floor(minLonIn1*60):topoFileInterval:ceil(maxLonIn1*60)];
xi = xi/60.;
xi = xi - centerLon;

xIndex1 = int32(floor(minLonIn2*60/topoFileInterval)+1);
xIndex2 = int32(ceil(maxLonIn2*60/topoFileInterval));

yi=[floor(minLatIn1*60):topoFileInterval:ceil(maxLatIn1*60)];
yi=yi/60;
yi = ((asinh(tan(yi*pi/180.))* 180.0 / pi));

yIndex1 = int32(floor(90+minLatIn1)*60/topoFileInterval+1);
yIndex2 = int32(ceil(90+maxLatIn1)*60/topoFileInterval+1);

fprintf('\n[INFO] xindex1, xindex2, yindex1, yindex2: %d, %d, %d, %d\n',xIndex1,xIndex2,yIndex1,yIndex2);
xStep = 1;
yStep = 1;
if(xIndex1 > xIndex2)
   xStep =-1;
end;
if(yIndex1 > yIndex2)
   yStep =-1;
end;


%
% adjustment to make sure vector lengths match and no error due to rounding or step size
%
xIndex2 = xIndex2-xStep*(length(xIndex1:xStep:xIndex2)-length(xi));
yIndex2 = yIndex2-yStep*(length(yIndex1:yStep:yIndex2)-length(yi));

zi= values(yIndex1:yStep:yIndex2,xIndex1:xStep:xIndex2);

load dotcolors % loads array cmap

if(debug)
   fprintf('\n[INFO] Pre-processing complete\n');
end;

%
% ------------------------------------------
% read seismograms, filter and resample
% ------------------------------------------
%
if(debug)
   fprintf('\n[INFO] read waveforms:%s/%s \n',dataPath,strcat('*',channel,'.*.SAC'));
end;

%[staName staCoord seisDataAll] = getSeis(ftype, filter, dataPath, tArray, strcat('*',channel,'.*.SAC'));
%
%staCount = 0;
%for i = 1:size(staCoord(:,1))
%   if(staCoord(i,1)>= minLonIn) && (staCoord(i,1)<= maxLonIn) && (staCoord(i,2)>= minLatIn) && (staCoord(i,2)<= maxLatIn)
%   staCount = staCount+1;
%   seisData(staCount,:) = seisDataAll(staCount,:);
%   lon(staCount) = staCoord(i,1);
%   lat(staCount) = staCoord(i,2);
%   end;
%end;
%[r c] = size(seisData);

[staName staCoord seisData] = getSeis(ftype, filter, dataPath, tArray, strcat('*',channel,'.*.SAC'));
%seisData = abs(hilbert(seisData));
[r c] = size(seisData);
lon = staCoord(:,1);
lat = staCoord(:,2);


%
% ------------------------------------------
% set movie parameters
% ------------------------------------------
%
% get distance to "center" of array
%
viewCount=0;
for i=1:length(lat)
   if(lat(i)>= minLatIn && lat(i) <= maxLatIn && lon(i) >= minLonIn && lon(i) <= maxLonIn)
      viewCount = viewCount+1;
      viewLat(viewCount) = lat(i);
      viewLon(viewCount) = lon(i);
   end;%if
end;%for

if(viewCount==0)
   viewLat = lat;
   viewLon = lon;
end;%if

medianStaLat = median(viewLat);
medianStaLon = median(viewLon);

if(isEventSet)
   if(debug)
      fprintf('[INFO] median(medianStaLat,medianStaLon,eventLat,eventLon) =(%5.3f,%5.3f,%5.3f,%5.3f\n)',medianStaLat,medianStaLon,eventLat,eventLon);
   end;
   [del az] = stadis2(medianStaLat,medianStaLon,eventLat,eventLon);
   if(debug)
      fprintf('[INFO] median(medianStaLat,medianStaLon,del,az) =(%5.3f,%5.3f,%5.3f,%5.3f\n)',medianStaLat,medianStaLon,del,az);
   end;

   %
   % compute some nominal travel times to array center
   %
   p1 = pSlowness * del; 
   r1 = rSlowness * del; 
   r2 = rSlowness * (360-del);

end;

%
% ------------------------------------------
% select and prepare reference seismogram for bottom of plot
% ------------------------------------------
%
% select seismogram in file # refIndex near the center
%
if(debug)
   fprintf('[INFO] %s\n','choosing new reference trace close to the "center" of the array');
end;

refIndex = 0; % index of the reference station
prevDel = 9999;
for i=1:length(lat)
   [delr azr] = stadis2(lat(i),lon(i),medianStaLat,medianStaLon);
    if (delr <= prevDel) && ~isnan(lon(i))
        refIndex    = i;
        prevDel  = delr;
    end
end

if refIndex < 1, return; end; % reference station was not found!!

%
% reference station data with respect to event
%
refLat  = lat(refIndex);
refLon  = lon(refIndex);
refSta  = strtrim(strcat(staName(refIndex,:)));

if(isEventSet)
   [del az] = stadis2(eventLat,eventLon,refLat,refLon);

   refAz   = az;
   refDel  = round(del);

   if(debug)
      fprintf('[INFO] ref sta :%s\n',refSta);
      fprintf('[INFO](reference station & distance:refIndex,refSta,refLat,refLon,round(refDel)): %d %s at %5.3f,%5.3f (%d)',refIndex,refSta,refLat,refLon,round(refDel));
   end;
end;

tStart(refIndex)=movieStartEpoch;
npts(refIndex)=length(seisData(refIndex,:));

is = max( [ceil( int32((movieStartEpoch - tArray(1)) / movieTStep )) + 1 1] );    % starting index of window
movieStartEpoch = tArray(1) + (is -1) * movieTStep;

ie = min( [floor( int32((movieEndEpoch   - movieStartEpoch) / movieTStep )) - 1 npts(refIndex)] ); % ending index of window
movieEndEpoch = movieStartEpoch + (ie -1) * movieTStep;

%
% store the desired portion of data
%
movieData = zeros(length(seisData(:,1)),ie-is+1);

%
% grab the reference station trace before scaling
%
refData = zeros(ie-is+1);
refData = seisData(refIndex,is:ie);

for i=1:length(seisData(:,1))
   movieData(i,:) = seisData(i,is:ie);
end;
movieData = scaleRows(movieData, magnification);%Each station is individually scaled to between [-magnification +magnification]

movieT = zeros(length(movieData(1,:))); % movie trace time since the event time
for p=is:ie
    movieT(p-is+1) = (p - 1)*movieTStep; % time scale for reference seismogram
end;

if(debug)
   fprintf('[INFO] movie start T: %d end T: %d',movieT(1),movieT(length(movieT)));
end;

%
% calculate phases
%
if(isEventSet)
   if(debug)
     fprintf('[INFO] eventDepth=%f,phaseList=%s,[refLat=%f refLon=%f],[eventLat=%f,eventLon=%f]\n',eventDepth,phaseList,refLat,refLon,eventLat,eventLon);
   end;

   phaseTime = taupTime('iasp91',eventDepth,phaseList,'sta',[refLat refLon],'evt',[eventLat,eventLon])

   if(debug)
      for i=1:length(phaseTime)
         fprintf('[INFO] phase: %s time: %f',phaseTime(i).phaseName, phaseTime(i).time);
      end;
   end;
end;

%
% When we plot the reference time series later on, there are certain
% circumstances, which I cannot entirely identify, which cause the x axis
% line not to plot if the time series points are the same as the Y axis
% limits. So, I enlarge the Y axis limits by 4% and that seems to fix the
% problem. Enlarging the limits by "eps" doesn't do the trick. Ughh!
%
smin = min(movieData(refIndex,:)); % min seismogram amplitude in window used for movie
smax = max(movieData(refIndex,:)); % max seismogram amplitude in window used for movie
smin = smin - 0.01 * abs(smin);
smax = smax + 0.01 * abs(smax);

if(debug)
   fprintf('[INFO] movie Data %d\n',movieData(refIndex,:));
   fprintf('[INFO] processed reference trace\n');
end;

%
% get information for station profile or cross section
%
%eqreg  = char(feregion(eventLat,eventLon));
%
%[iy,id,ih,im,ss] = ep2dat(eventTime); % use earthquake time to get year and day
%h1 = datestr(datenum([double(iy) 1 double(id)]),'mmmm dd, yyyy, ');
%
%%
%% Sample of target appearance of title
%% January 15, 2009, east of the Kuril Islands, M=7.3
%%
%h1 = char([h1 eqreg]);
%headline = sprintf('%s%s%2.1f  ',h1,', M=',eventMag);
%
%     myaa: Renders an anti-aliased version of the current figure.
%
%myaa();

headline=plotTitle;

%
% ------------------------------------------
% ADJUST COORDINATES TO MERCATOR PROJECTION (centered on the array "center" longitude)
%
%
% process data points for scatter plopt
%
% ------------------------------------------

%
% stations
%
if(debug)
  fprintf('\n[INFO] lon - centerLon:%d\n',size(centerLon));
end;

lon = lon - centerLon;
lat = asinh(tan(lat*pi/180.))* 180.0 / pi;

%
% boundaries
%
shore.data(:,1) = shore.data(:,1) - centerLon;
shore.data(:,2) = asinh(tan(shore.data(:,2)*pi/180.))* 180.0 / pi;
boundary.data(:,1) = boundary.data(:,1) - centerLon;
boundary.data(:,2) = asinh(tan(boundary.data(:,2)*pi/180.))* 180.0 / pi;
political.data(:,1) = political.data(:,1) - centerLon;
political.data(:,2) = asinh(tan(political.data(:,2)*pi/180.)) * 180.0 / pi;

%
% topo
%
%xi = xi - centerLon;
%yi = asinh(tan(yi*pi/180.0))* 180.0 / pi;

%
% ------------------------------------------
% Open figure and plot static part of movie
% ------------------------------------------
%
%fig_h  = figure('visible','off'); %turns visibility of figure off
fig_h  = figure(); %turns visibility of figure off
set (fig_h,'Position',[1 150 screenWidth screenHeight],'Color',[1 1 1],'Units','pixels');

%set(gca, 'Position', get(gca, 'OuterPosition') - ...
%    get(gca, 'TightInset') * [-1 0 1 0; 0 -1 0 1; 0 0 1 0; 0 0 0 1]);

%tig = get(gca,'tightinset');
%set(gca,'position',[tig(1) tig(2) 1-tig(1)-tig(3) 1-tig(2)-tig(4)]);
set (fig_h,'RendererMode','manual','Renderer','zbuffer');

% create the base map
subplot(4,1,1:3);
set(gca,'Color',[1 1 1 ]);
hold on

fprintf('\n[INFO] x,y,Z,%d, %d, %d\n',length(xi),length(yi),size(zi));
%contourf(xi,yi,zi,[0 0],'EdgeColor','none'); % shade the continent with a nice grey color
fprintf('\n[INFO] contourf [xi],[yi],[zi]: [%d, %d], [%d, %d], [%d %d]\n',xi(1),xi(length(xi)), yi(1),yi(length(yi)),zi(1),zi(size(zi)));

contourf(xi,yi,zi,[-10000 0],'EdgeColor','blue'); % shade the continent with a nice grey color
colormap(cmap2);

plot(political.data(:,1),political.data(:,2),'k');
%plot(shore.data(:,1),shore.data(:,2),'k');
plot(boundary.data(:,1),boundary.data(:,2),'k');


%
% the fill above masks the Great Lakes, now put the Great Lakes back with blue color
%
load conus;
gtlon=gtlakelon-centerLon;
gtlat= asinh(tan(gtlakelat*pi/180.))* 180.0 / pi;

% lakes are separated by NaN
% Use polysplit to create two cell arrays, latc and lonc
%
[latc,lonc]=polysplit(gtlat,gtlon);
fill(lonc{1},latc{1},'w');
fill(lonc{2},latc{2},'w');
fill(lonc{3},latc{3},'w');

%
% limits for projected coordinates
%
xlim([minLon maxLon]);
if(debug)
   fprintf('[INFO] min & max lon(%d %d)\n',minLon,maxLon);
end;

ylim([minLat maxLat]);
if(debug)
   fprintf('[INFO] min & max lat(%d %d)\n',minLat,maxLat);
end;

%set(gca,'Box','on','XTick',[],'YTick',[],'DataAspectRatio',[1 aspectRatio 1])
set(gca,'Box','on','XTick',[],'YTick',[]);
title(headline,'FontSize',headerFontSize,'FontWeight','bold');

if(debug)
   fprintf('[INFO] visualization x limits: %5.3f , %5.3f\n',xlim);
   fprintf('[INFO] visualization y limits: %5.3f , %5.3f\n',ylim);
end;

%
% calculate the great circle path
%
if(isEventSet)
   [distgc, azgc] = distance(eventLat,eventLon,refLat,refLon);
   %azgc = azimuth(eventLat,eventLon,refLat,refLon);

   if(debug)
      fprintf('[INFO] (distgc, azgc): (%5.3f , %5.3f)\n',distgc, azgc);
   end;

   [latgc,longc] = track1('gc',eventLat,eventLon,azgc,distgc);

   if(debug)
      fprintf('[INFO] XYGC count: %d\n',length(latgc));
      for i=1:length(latgc)
         fprintf('[INFO] latGC: %5.3f lonGC: %5.3f\n',latgc(i),longc(i));
      end;
   end;

   %
   % here we extract the GC path that falls within the view port
   %
   visibleLonGc = [];
   visibleLatGc = [];
   visibleCount=0;
   for i=1:length(latgc)
      longc(i) = longc(i) - centerLon;
      latgc(i) = asinh(tan(latgc(i)*pi/180.))* 180.0 / pi;
      if((latgc(i) >= minViewLat & latgc(i) <= maxViewLat) & (longc(i) >=minViewLon & longc(i) <= maxViewLon))
         visibleCount = visibleCount +1;
         visibleLonGc(visibleCount) = longc(i);
         visibleLatGc(visibleCount) = latgc(i);
         if(debug)
            fprintf('[INFO] visibleLatGc: %5.3f visbleLonGc: %5.3f\n',visibleLatGc(visibleCount),visibleLonGc(visibleCount));
         end;
      end;
   end;
end;

%
% z motion color key
%
%text(maxViewLon - centerLon,asinh(tan((minViewLat+10)*pi/180.))* 180.0 / pi,...
%        'motion up',...
%        'HorizontalAlignment','right',...
%        'BackgroundColor',[1 0 0],...
%        'Margin',10);
%
%text(maxViewLon - centerLon,asinh(tan((minViewLat+5)*pi/180.))* 180.0 / pi,...
%	'motion down',...
%	'HorizontalAlignment','right',... 
%	'BackgroundColor',[0 0 1],...
%	'Margin',10);

%rheadLon = maxViewLon-5 + centerLon;
%rheadLat = asinh(tan((minViewLat+5)*pi/180.))* 180.0 / pi;
%rtailLat = asinh(tan((minViewLat+4)*pi/180.))* 180.0 / pi;
%rtailLon = rheadLon;
%arrowhead([rtailLon,rheadLon],[rtailLat,rheadLat],'r',[],3);
%
%bheadLon = rheadLon;
%bheadLat = asinh(tan((minViewLat)*pi/180.))* 180.0 / pi;
%btailLat = asinh(tan((minViewLat+1)*pi/180.))* 180.0 / pi;
%btailLon = bheadLon;
%arrowhead([btailLon,bheadLon],[btailLat,bheadLat],'b',[],3);
%
%text(rheadLon,asinh(tan((minViewLat+5.6)*pi/180.))* 180.0 / pi,'up','FontSize',8,'HorizontalAlignment','center');
%text(rheadLon,asinh(tan((minViewLat+2.4)*pi/180.))* 180.0 / pi,'vertical motion','FontSize',10,'HorizontalAlignment','center');
%text(bheadLon,asinh(tan((minViewLat-0.2)*pi/180.))* 180.0 / pi,'down','FontSize',8,'HorizontalAlignment','center');
%text(maxViewLon-49.5+centerLon,asinh(tan((minViewLat-0.2)*pi/180.))* 180.0 / pi,'www.iris.edu/dms/products/usarraygmv','FontSize',8,'HorizontalAlignment','center');

%
% ------------------------------------------
% plot the seismogram
% ------------------------------------------
%
subplot(4,1,4:4);
hold on
sampStart = 1;

%
% Z-component here we assume that the unit is m/s (true for TA stations) but should be made more generic
%
%if(strcmp(technology,'SEIS'))
%   [ dispT, dispData , dispTitle, smin, smax] = traceInt(movieT(:,1),refData);
%else
% dispT = movieT;
% dispData = refData;
% dispTitle = 'Pascal';
%end;
%fprintf('[INFO] seisData %d, %d\n',min(seisData(refIndex,is:ie)),max(seisData(refIndex,is:ie)));
%plot ( movieT,seisData(refIndex,is:ie),'LineWidth',2, 'LineSmoothing','on','color','g' );
%fprintf('[INFO] refData %d, %d\n',min(refData),max(refData));
plot ( movieT,movieData(refIndex,:),'LineWidth',1, 'LineSmoothing','on','color','k' );
%fprintf('[INFO] dispData %d, %d\n',min(dispData),max(dispData));
%plot ( movieT,dispData,'LineWidth',1, 'LineSmoothing','on','color','k' );
text(movieT(5),0.1,strcat(refSta,'/',channel),'FontSize',10,'VerticalAlignment','bottom');


fprintf('[INFO] trace smin , smax %d , %d\n',smin,smax);
plot ( [movieT(1) movieT(length(movieT))], [smin smin], 'k' );
xlim(gca, [movieT(1) movieT(length(movieT))]);
ylim(gca, [smin smax*1.25]);

%
% trace label
%
tLabel = ' ';
if(isEventSet)
   tLabel = sprintf('%s%5.1f%1s/%d km  Azimuth %5.1f%s%5.1f%1s%5.1f','  Distance ',refDel,'\circ',round(deg2km(refDel)),refAz,'\circ' );
end;

if(debug)
   fprintf('[INFO] trace label tmin, smax, tLabel %d , %d, %s\n',movieT(1),smax,tLabel);
end;

%text(double(movieT(1)+(movieT(length(movieT))-movieT(1))/2.3),double(1.36*smax),tLabel,'FontSize',10,'VerticalAlignment','bottom');

if(debug)
   fprintf('[INFO] trace tmin, tmax %d , %d, %s\n',movieT(1),movieT(length(movieT)));
end;

%
% time axis label
%
xlabel('Visualization Time (hh:mm:ss)','FontSize',12,'FontWeight','bold');

%
% find the max for normalization
%
%maxVertical   = max(dispData);
%
% amplitude scale to the right of the Z component
%
%plot ([dispT(length(dispT)),dispT(length(dispT))],[0,smax],'Color','blue','LineWidth',4); % the blue scale line
%scaleHeight = round(maxVertical);
%scaleText = text(dispT(length(dispT)-3), smax*0.8, [num2str(scaleHeight),' ',dispTitle,'   '], 'Color', 'blue','fontsize',11,'LineWidth',5,'Interpreter','latex');

%set(scaleText,'Rotation',0);
%set(scaleText,'HorizontalAlignment','right','VerticalAlignment','middle');

%
% direction legend
%
%arrowhead([movieT(length(movieT)-10)  movieT(length(movieT)-4)],[0.4*smax,smax],'r',[],3);
%text(movieT(length(movieT)-2),smax*0.5,'vertical motion','FontSize',8,'VerticalAlignment','middle','HorizontalAlignment','right');
%arrowhead([movieT(length(movieT)-10) movieT(length(movieT)-4)],[0.4*smax,0],'b',[],3);
%arrowhead([movieT(length(movieT)-15)  movieT(length(movieT)-15)],[0.25*smax,1.25*smax],'r',[],3);
%arrowhead([movieT(length(movieT)-10) movieT(length(movieT)-10)],[1.25*smax,1.0*smax],'b',[],3);
xs = movieT(length(3));
ys = 1.6*smax;
text(xs,ys,['\fontsize{9}vertical motion   {\fontsize{120}\color{red}.\fontsize{9}up   \fontsize{120}\color{blue}.\fontsize{9}down}'])

%text(xs-5,smax*1.2,'vertical motion','FontSize',8,'VerticalAlignment','middle','HorizontalAlignment','right');
%plot(xs,ys,'o','MarkerEdgeColor','none','MarkerFaceColor','r');
%text(xs+2,ys,'up','FontSize',8,'VerticalAlignment','middle','HorizontalAlignment','left');
%plot(xs+4,ys,'o','MarkerEdgeColor','none','MarkerFaceColor','b');
%text(xs+6,ys,'down','FontSize',8,'VerticalAlignment','middle','HorizontalAlignment','left');

%yCount = 3;
%
%yTicks = zeros(yCount);
%yTicks = [smin 0 smax];
%yTickLabels = {};
%for i=1:length(yTicks)
%   yTickLabels{i}  = sprintf('%4.0f',yTicks(i));
%end;

%
% do not plot y-axis
%
set(gca,'YTick',[],'YColor','w');

%
% set the amplitude axis (no limit change, just the labels)
%
%
%set(gca,'TickDir','out');
%set(gca,'YTick', yTicks); % tick locations
%set(gca,'YTickLabel', yTickLabels); % label each tick with a string
%
%ylabel(dispTitle,'FontSize',10,'FontWeight','bold');
%
% reduce the axis label distance from the axis (in data units, sec)
%
%ylabh = get(gca,'YLabel');
%set(ylabh,'Position',get(ylabh,'Position') + [(movieT(length(movieT))-movieT(1))/40 0 0]) ;
%
%
% to have regular tick labels at every xTick seconds
%
tmin = ceil(1.0*movieT(1)/xTick)*xTick;
tmax = floor(1.0*movieT(length(movieT))/xTick)*xTick;
mCount = 1 + (tmax - tmin)/xTick;

if(debug)
   fprintf('[INFO] tmin tmax mCount %d , %d, %d\n',tmin,tmax,mCount);
end;

ticks = zeros(mCount);
tickValue = tmin;
for i=1:mCount
   ticks(i) = tickValue;
   tickValue = tickValue + xTick;
   tickCount = i;
end;

%
% relabel the time ticks
%
%xTicks = get(gca,'XTick'); % get the x values for each tick mark
xTicks = ticks;

if(debug)
   fprintf('[INFO] time axis tick values: %d\n',xTicks);
end;

xTickLabels = {};
doSkip = 0; % we want to plot every other label
for i=1:length(xTicks)
   if(doSkip <=0)
      hLabel = idivide(xTicks(i),int32(3600));
      mLabel = idivide(mod(xTicks(i),3600),int32(60));
      sLabel = xTicks(i)-hLabel*3600-mLabel*60;
      xTickLabels{i}  = sprintf('%02d:%02d:%02d',hLabel,mLabel,sLabel);

      if(debug)
         fprintf('[INFO] time axis tick labels: %s\n',xTickLabels{i});
      end;

      doSkip = 1;
   else
      xTickLabels{i} = ' ';
      doSkip = 0;
   end;
   
end;

%
%set the time axis (no limit change, just the labels)
%
set(gca,'XTick', [ticks(1):xTick:ticks(tickCount)]); % tick locations
set(gca,'XTickLabel', xTickLabels); % label each tick with a string

%
% ------------------------------------------
% now, plot the phases
% ------------------------------------------
%
oldPhaseTime=-999;
shift=1;

if(isEventSet)
   for i=1:length(phaseTime)
      phaseEpoch = eventEpoch+phaseTime(i).time;
      if(phaseEpoch < movieStartEpoch || phaseEpoch > movieEndEpoch)
         continue;
      else
         phaseT = double(phaseEpoch - movieStartEpoch);
      end;

      fprintf('[INFO] phase time #%d: %d %s\n',i,phaseTime(i).time,phaseTime(i).phaseName);
      fprintf('[INFO] phase oldtime time spacing #%d: %d %d %d\n',i,oldPhaseTime,phaseSpacing);
      if(i > 1)
         if(~strcmp(phaseTime(i).phaseName, phaseTime(i-1).phaseName) && (phaseT-oldPhaseTime) > phaseSpacing)
            plot ([phaseT,phaseT],[smin,smax*0.8],'blue'); % the blue phase line
            text(phaseT, smax*shift, phaseTime(i).phaseName, 'Color', 'blue','fontsize',7);
            oldPhaseTime = phaseT;

            % shift phase vertical alignment for a better readbility
            if(shift == 1)
               shift=1.1;
            else
               shift=1;
            end
         end;
      else
         plot ([phaseT,phaseT],[smin,smax*0.8],'blue'); % the blue phase line
         text(phaseT, smax*shift, phaseTime(i).phaseName, 'Color', 'blue','fontsize',7);
         oldPhaseTime = phaseT;
         % shift phase vertical alignment for a better readbility
         if(shift == 1)
            shift=1.1;
         else
            shift=1;
         end
      end;
   end;
end

%
% R1 & R2 & I
%
if(isEventSet)
   shift=1.1;
   r1Time = (distgc * rSlowness);
   r1Epoch = eventEpoch+r1Time;
   fprintf('[INFO] r1 phase time #%d %d %d\n',r1Time,r1Epoch,eventEpoch);
   if(r1Epoch >= movieStartEpoch && r1Epoch <= movieEndEpoch) 
      r1T = double(r1Epoch - movieStartEpoch);
      fprintf('[INFO] r1Epoch movieStartEpoch r1T %d %d %d\n',r1Epoch,movieStartEpoch,r1T);
      if(del > 20)
           plot ([r1T,r1T],[smin,smax*0.8],'blue'); % the blue phase line
           text(r1T, smax*shift, 'R1', 'Color', 'blue','fontsize',7);
      end;
   end

   r2Time = ((360 - distgc) * rSlowness);
   r2Epoch = eventEpoch+r2Time;
   if(r2Epoch >= movieStartEpoch && r2Epoch <= movieEndEpoch) 
      r2T = double(r2Epoch - movieStartEpoch);
      if(del > 20)
           plot ([r2T,r2T],[smin,smax*0.8],'blue'); % the blue phase line
           text(r2T, smax*shift, 'R2', 'Color', 'blue','fontsize',7);
      end;
   end;

   if(strcmp(technology,'INFRA'))
      iTime = distgc * iSlowness;
      if( iTime < tmax)
           plot ([iTime,iTime],[smin,smax*0.8],'blue'); % the blue phase line
           text(r2Time, smax*shift, 'I', 'Color', 'blue','fontsize',7);
      end;
   end;

end;
%
% set up output AVI file name
%
[iy,id,ih,im,ss] = ep2dat(eventTime);

mname = sprintf('%s/%s',moviePath,ename);

if(debug)
   fprintf('[INFO] visualization file: %s\n',mname);
end;

%
% Web link file
%

[iy,id,ih,im,ss] = ep2dat(movieStartEpoch);
dateNum = (datenum([1970 1 1 0 0 0])*86400 + double(movieStartEpoch)) / 86400;
eqDateTime =datestr((datenum([1970 1 1 0 0 0])*86400 + double(eventTime)) / 86400, 'yyyy/mm/dd HH:MM:SS');
startDateTime =datestr((datenum([1970 1 1 0 0 0])*86400 + double(movieStartEpoch)) / 86400, 'yyyy/mm/dd HH:MM:SS');
endDateTime =datestr((datenum([1970 1 1 0 0 0])*86400 + double(movieEndEpoch)) / 86400, 'yyyy/mm/dd HH:MM:SS');

%
% here we want to create an AVI movie without displaying the figure!
%
aviobj = avifile(mname, 'compression', 'none', 'fps', 25); % create AVI file

% Main loop over frames of movie, extracting time time slice from network
% on each loop
%
frame = 0;
frameTotal = 0;
frameTrip = 0;
t = movieStartEpoch - movieTStep;

%
% one movie frame at a time
%
fprintf('\n[INFO]FRAME 0');


for tStep=1:length(movieT)
    t = movieStartEpoch + (tStep-1) * movieTStep;
    frame = frame + 1;
    
    %
    % do all stations for this time slot
    %
    for staIndex=1:length(movieData(:,tStep))

        %MBi = IX(j);     % go through files in time sort order
        %MBtdiff = ( movieStart - tStart(i) ) / dt(i);
        %MBm = int32(tdiff) + 1;  % index into seismogram
        %MBif m<1 || m>npts(i)
        %MB    amp(i)= 0.0 ; % seismogram does not overlap target time sample - set value to zero
        %MBelse
        %MB    amp(i) = sacdat(m,i);
        %MBend;

        %
        % Select color for this station-amplitude value.
        % Colormap has 64 values going from negative to positive, so map amplitude to a 1-to-64 scale
        % We assume peak-to-peak amplitudes in seismograms are +-1 - so amplitudes outside this range
        % will saturate color scale.
        % If amp is outside of +-1 then color scale saturates (we may have
        % scaled seismogram to values larger than +-1, so that small
        % amplitudes will be visibile.
        % 
        %
        % manoch - 2010-11-26 added check on NaN to avoid plotting blue dots when we only have waveform segments
        % 
        if isnan(movieData(staIndex,tStep))
           z = 32;
        else
           z = int32(32.5 * movieData(staIndex,tStep) + 31.5);
        end;

        if(debug)
           fprintf('[INFO] Z:%d\n',z);
        end;

        if z<1
            z=1;
        elseif z>64
            z=64;
        end;
        col(staIndex,1:3) = cmap(z,1:3);

        if(debug)
           fprintf('[INFO] Color:%d %d %d\n',col(staIndex,1:3));
        end;
    end;
    %
    if frameTrip > 0
        delete(h1);
        delete(h2);
        delete(h3);
        delete(h4);
    end;
    frameTrip = frameTrip + 1;

    %
    % plot the station dots
    %
    fprintf('.');
    subplot(4,1,1:3);
    h1=scatter(lon,lat,markerSize,col,'filled','MarkerEdgeColor',markerEdgeColor);
    h2=scatter(lon(refIndex),lat(refIndex),refMarkerSize,circleColor); % yellow circle around ref station
    set(h2,'LineWidth',circleThickness); % make circle even thicker!

    %
    % calculate the great circle path (we do it for each frame to make sure that it is not covered by other points
    %
    %plot(visibleLonGc,visibleLatGc,'y-','LineWidth',1);
    %
    % plot the event location. Because of the view port, this will only be visible for the US events
    %
if(isEventSet)
    plot(eventLon - centerLon,asinh(tan(eventLat*pi/180.))* 180.0 / pi,'rp',...
       'MarkerSize',30,'MarkerFaceColor','r');
end;
  
if(showGCPath)
    plot(visibleLonGc,visibleLatGc,'y-','LineWidth',1);
    %plot(visibleLonGc(1:int32(end/9)),visibleLatGc(1:int32(end/9)),'k','LineWidth',1);
    %arrowhead([visibleLonGc(int32((end/9)-int32(end/18))),visibleLonGc(int32(end/9))],[visibleLatGc(int32((end/9)-int32(end/18))),visibleLatGc(int32(end/9))],'k',[],3);
 
    %plot(visibleLonGc(int32(2*end/9):int32(3*end/9)),visibleLatGc(int32(2*end/9):int32(3*end/9)),'k','LineWidth',1);
    %arrowhead([visibleLonGc(int32((3*end/9)-int32(end/18))),visibleLonGc(int32(3*end/9))],[visibleLatGc(int32((3*end/9)-int32(end/18))),visibleLatGc(int32(3*end/9))],'k',[],3);
 
    %plot(visibleLonGc(int32(4*end/9):int32(5*end/9)),visibleLatGc(int32(4*end/9):int32(5*end/9)),'k','LineWidth',1);
    %arrowhead([visibleLonGc(int32((5*end/9)-int32(end/18))),visibleLonGc(int32(5*end/9))],[visibleLatGc(int32((5*end/9)-int32(end/18))),visibleLatGc(int32(5*end/9))],'k',[],3);
 
    %plot(visibleLonGc(int32(6*end/9):int32(7*end/9)),visibleLatGc(int32(6*end/9):int32(7*end/9)),'k','LineWidth',1);
    %arrowhead([visibleLonGc(int32((7*end/9)-int32(end/18))),visibleLonGc(int32(7*end/9))],[visibleLatGc(int32((7*end/9)-int32(end/18))),visibleLatGc(int32(7*end/9))],'k',[],3);
 
    %plot(visibleLonGc(int32(8*end/9):int32(9*end/9)),visibleLatGc(int32(8*end/9):int32(9*end/9)),'k','LineWidth',1);
    %arrowhead([visibleLonGc(int32((9*end/9)-int32(end/18))),visibleLonGc(int32(9*end/9))],[visibleLatGc(int32((9*end/9)-int32(end/18))),visibleLatGc(int32(9*end/9))],'k',[],3);
end;
    %arrow([longc(end-3),latgc(end-3)],[longc(end),latgc(end)],'y');

if(isEventSet && showCC)
   %
   %  Add concentric circles around epicenter to show wavefront arcs
   %
   fakeLat = [minLatIn, minLatIn, maxLatIn, maxLatIn];
   fakeLon = [minLonIn, maxLonIn, maxLonIn, minLonIn];
   sub_circle(eventLat,eventLon,centerLon,fakeLat,fakeLon)
end;

    %
    % plot the time stamp
    [iy,id,ih,im,ss] = ep2dat(t);
    %
    % change the time stamp from Julian day to regular date
    % 
    dateNum = (datenum([1970 1 1 0 0 0])*86400 + double(t)) / 86400;
    %traceTitleString = strcat(datestr(dateNum, 'yyyy/mm/dd HH:MM:SS'),' UTC (',num2str(round(movieT(tStep))),' s)',refSta,'(',chan,')',tLabel);
    %traceTitleString = sprintf('%s%s%s%s  %s%s%s',datestr(dateNum, 'yyyy/mm/dd HH:MM:SS'),' UTC (',num2str(round(movieT(tStep))),' s)',tLabel);
    traceTitleString = sprintf('%s%s%s%s  %s%s%s',datestr(dateNum, 'yyyy/mm/dd HH:MM:SS'),' UTC (',num2str(round(movieT(tStep))),' s)');
    h3=text(minLon+timeHorizontalOffset,minLat+timeVerticalOffset,[traceTitleString,filterString,dtString],'FontSize',12);

    %
    % now, plot the moving red line on the seismogram
    subplot(4,1,4:4);
    h4=plot ([movieT(tStep),movieT(tStep)],[smin,smax],'red'); % the red moving line

    %
    % IRIS logo
    %
    %axis equal;
    %axis manual;
    %was = axes('Position',irisUrlPosition);
    %text(0,irisLogoHeight/2.0,'www.iris.edu/dms/products/usarraygmv','FontSize',8,'HorizontalAlignment','left');
    %axis off;

    has = axes('Position',irisLogoPosition);
    I=imread(irisLogo);
    image(I);
    daspect([1 1 1]);
    axis off;

    %
    % grab the frame, increment the time step, and loop around
    %
    pause(0.01);
    F(frame) = getframe(gcf);
    if frame > frameFlushInterval
        fprintf(' >>>>>');
        if(debug)
           fprintf('[INFO] flushing frames\n');
        end;
        aviobj = addframe(aviobj,F);
        frameTotal = frameTotal + frame -1;

        if(debug)
           fprintf('[INFO] frames total: %d\n',frameTotal-1);
        end;

        fprintf('\n[INFO]FRAME %d ',frameTotal);
        frame = 0;
    end
    t = t + movieTStep;
end;

if frame > 0
    frameTotal = frameTotal + frame -1;
    fprintf(' >>>>>');
    if(debug)
       fprintf('[INFO] flushing the %d remaining frames\n',frame);
    end;
    aviobj = addframe(aviobj,F(1:frame));
        
    if(debug)
       fprintf('[INFO] frames total: %d\n',frameTotal-1);
    end;
end;

pause(1);
aviobj     = close(aviobj);% closes the AVI file
status_fig = close(gcf);
pause(5);
if(debug)
   fprintf('[INFO] total frames: %d\n',frameTotal);
end;
clear F sacdat sachdr xi yi zi lat lon shore boundary political npts;
pause(0.5);
