function [stName, stCoord, seisData] = getSeis(ftype, f, sacPath, t,filePattern)
%
%=======================================================================================
%
% Copyright (C) 2011  Product Team, IRIS Data Management Center
%
%    This is a free software; you can redistribute it and/or modify
%    it under the terms of the GNU Lesser General Public License as
%    published by the Free Software Foundation; either version 3 of the
%    License, or (at your option) any later version.
%
%    This script is distributed in the hope that it will be useful, but
%    WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%    Lesser General Public License (GNU-LGPL) for more details.  The
%    GNU-LGPL and further information can be found here:
%    http://www.gnu.org/
%
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%
% Description:
%  OPTIMIZED FOR ANIMATIONS NOT FOR DATA COLLECTION!
%
% Input:
%   ~Frequency (f): [Lower Bound, Upper Bound]
%   ~Sac File Path (sacPath): SAC File Location
%   ~Epoch Time (t): [Start Time, End Time, Time Step]
%
% Output:
%   ~Station Coordinates (stCoord): [Longitude Latitude]
%   ~Seismic Data (seisData):  [s1(t1) s1(t2) ... s1(tn)
%                               s2(t1) s2(t2) ... s2(tn)
%                               :
%                               sk(t1) sk(t2) ... sk(tn)]
%
% Function Dependencies:
%   ~rsac (Ammon):
%   ~seisInsert (Lloyd):
%   ~seisClean (Lloyd):
%
% History:
%       2010-07-16 MB: added filePattern in preparation for 3D data
%       2010-05-10 MB: had to add double() since it appears that MATLAB is doing integer
%                          operation and is losing deciamal values!!! Verified by testing but could not
%                          find the cause
%       2009-08-29 Andrew Lloyd
%    
%=======================================================================================

dirList = dir(sacPath);
dirList = dir(strcat(sacPath,'/',filePattern));
num1 = length(dirList);
num2 = double(floor((double(t(2)) - double(t(1)))/double(t(3))));
numC = 0; %Number of SAC Files Combined
numD = 0; %Number of SAC Files Deleted

seisData = NaN(num1, num2);
stCoord = NaN(num1,2);
stName  = NaN(num1,10);
%fprintf('stCoord size %dx%d\n', size(stCoord));

%Traverses the Directory List
for i = 1:num1
    tmp = rsac2(dirList(i).name,sacPath);
    
    id = find(tmp.stla == stCoord(:,2) & tmp.stlo == stCoord(:,1));
    if(~isempty(id))
        tmp2 = rsac2(dirList(id).name,sacPath);
        
        if(tmp.kstnm == tmp2.kstnm)
            seisData(id,:) = seisInsert(ftype, f, seisData(id,:), t, tmp);
            
            fprintf('SAC files for %s have been combined.\n', tmp.kstnm);
            numC = numC + 1;
        else
            stName(id) = NaN;
            stCoord(id,:) = NaN;
            seisData(id,:) = NaN;
            
            numD = numD + 2;
        end %endIf
        
    else
        stName(i,:) = rPaddString(tmp.kstnm,10);
        %fprintf('%s\n',strcat(stName(i,1:10)));
        stCoord(i,:) = [tmp.stlo tmp.stla];
        seisData(i,:) = seisInsert(ftype, f, seisData(i,:), t, tmp);
    end %endIf
end %endFor

[stName stCoord seisData] = seisClean(stName, stCoord, seisData);

fprintf('%d SAC files with identical Lat/Lon have been combined.\n', numC);
fprintf('%d SAC files without identical Lat/Lon have been excluded.\n', numD);
