function [seisData] = seisFilterBW(ftype, f, seisData)
%
%=======================================================================================
%
% Copyright (C) 2011  Product Team, IRIS Data Management Center
%
%    This is a free software; you can redistribute it and/or modify
%    it under the terms of the GNU Lesser General Public License as
%    published by the Free Software Foundation; either version 3 of the
%    License, or (at your option) any later version.
%
%    This script is distributed in the hope that it will be useful, but
%    WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%    Lesser General Public License (GNU-LGPL) for more details.  The
%    GNU-LGPL and further information can be found here:
%    http://www.gnu.org/
%
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Description:
%   Tappers 5% off each end of the seismogram and performs a high and low pass
%   buttersworth filter.
%
% Inputs:
%   ~Filter type: (ftype)
%   ~Frequency (f): [Lower Bound, Upper Bound]
%   ~Unfiltered Seismic Data (seisData):
%
% Output:
%   ~Filtered Seismic Data (seisData)
%
% Function Dependencies:
%
% History:
%  2010-08-2 MB: now supports filter type
%  2009-08-29 Andrew Lloyd
%
%=======================================================================================
% 
if(~isnan(f(1)) & ~isnan(f(2)))
   if(strcmp(ftype,'HP') | strcmp(ftype,'hp') | strcmp(ftype,'BP') | strcmp(ftype,'bp') | strcmp(ftype,'LP') | strcmp(ftype,'lp'))
      npts = seisData.npts;
      nq = .5/seisData.dt; %Nyquist Frequency
      num1 = 0.05; %Taper Ratio
      %fprintf('[seisFilterBW] nq:%d dt:%d\n',nq,seisData.dt);

      %Taper
      t = tukeywin(npts, num1);
      seisData.d = t .* seisData.d;
      %fprintf('[seisFilterBW] Tapered:%d %d\n',min(seisData.d),max(seisData.d));
      %endTaper

      ftype = ftype;
      f1    = f(1);
      f2    = f(2);
      order = 4;
      %
      % 4th order is causing end effect for data with sampling of 100Hz and higher! Investigating
      % priblem was observed when worked on the China (CB network
      %
      if(seisData.dt <= 0.01)
         order=3;
      end;

      %Filter
      if(strcmp(ftype,'HP') | strcmp(ftype,'hp') | strcmp(ftype,'BP') | strcmp(ftype,'bp') )
         fband = f1/nq;
         if (fband >=1)
            fband = 0.99;
         elseif(fband <=0)
            fband = 0.0001;
         end;
         [b a] = butter(order, fband, 'high');
         seisData.d(1:npts) = filter(b, a, seisData.d(1:npts));
         %fprintf('[seisFilterBW] filter:%s %d %d %d %d %d\n',ftype,f1,f1/nq,order);
      end;

      if(strcmp(ftype,'LP') | strcmp(ftype,'lp') | strcmp(ftype,'BP') | strcmp(ftype,'bp') )
         fband = f2/nq;
         if (fband >=1)
            fband = 0.99;
         elseif(fband <=0)
            fband = 0.0001;
         end;
         [b a] = butter(order, fband, 'low');
         seisData.d(1:npts) = filter(b, a, seisData.d(1:npts));
         %fprintf('[seisFilterBW] filter:%s %d %d %d %d %d\n',ftype,f2,f2/nq,order);
      end;
   end;
end;
%endFilter
