function [ seisData ] = seisInsert(ftype,f, seisData, t, tmp)
%=======================================================================================
%
% Copyright (C) 2011  Product Team, IRIS Data Management Center
%
%    This is a free software; you can redistribute it and/or modify
%    it under the terms of the GNU Lesser General Public License as
%    published by the Free Software Foundation; either version 3 of the
%    License, or (at your option) any later version.
%
%    This script is distributed in the hope that it will be useful, but
%    WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%    Lesser General Public License (GNU-LGPL) for more details.  The
%    GNU-LGPL and further information can be found here:
%    http://www.gnu.org/
%
%    You should have received a copy of the GNU Affero General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Description:
%   Inserts seismic data into array where each column corresponds to a
%   specific time.  
%
% !The Following must be TRUE!
%   !(t1 - t2)%tstep = 0!
%   !lcm(tstep, dt) = tstep (where dt is the sample rate of the requested data)!
%   !lcm(1,dt) = 1!
%
% Input:
%   ~Frequency (f): [Lower Bound, Upper Bound]
%   ~Seismic Data (seisData): A Single Seismogram that may or may not be
%                             empty.
%   ~Epoch Time (t): [Start Time, End Time, Time Step]
%   ~Sac Structure (tmp):
%
% Output:
%   ~Seismic Data (seisData):
%
% Function Dependencies:
%   ~epoch (Woodward):
%   ~srmean (Ammon):
%   ~seisFilterBW (Lloyd):
%
% History:
%   2010-05-10 MB: had to add double() since it appears that MATLAB is doing integer
%                          operation and is losing deciamal values!!! Verified by testing but could not
%                          find the cause
%   2009-08-29 Andrew Lloyd
%
%=======================================================================================


num1 = length(seisData);
tmpStart = double(epoch(tmp.nzyear, tmp.nzjday, tmp.nzhour, tmp.nzmin,tmp.nzsec));
tmpEnd = double(tmpStart + tmp.npts*tmp.dt);
t1= double(t(1));
t2= double(t(2));
t3= double(t(3));

%
%
% this is a temporary fix to the problem of missing scale
% field caused by the new fetch data
% manoch 2010-07-22
%
% the new fetchData reports the scale. Now using scale 
% manoch 2011-01-10
%
%tmp.d = (tmp.d*10^9)/tmp.scale; %Converts to Units of nm/s
% temporary fix for an unknow problem that causes the scale to be set to -12345 if it is missing from the header!
% manoch 2011-01-11
%
if(tmp.scale == -12345.0)
   tmp.scale = 1.0;
end;

tmp.d = (tmp.d)/tmp.scale; % no conversion
%fprintf ('seisInsert[INFO] SCALE - %f\n',tmp.scale);
%fprintf ('scale[INFO] %d\n',tmp.scale);
%fprintf ('seisInsert[INFO] WARN - scaling is off!\n');
%fprintf ('dt[INFO] %d\n',tmp.dt);

%Indexing
if (t1 < tmpEnd && t2 > tmpStart)  
    if (t1 >= tmpStart)
        seisStart = 1;
        start = 1 + (t1 - tmpStart)/tmp.dt;       
    else
        seisStart = int32(1 + ceil((tmpStart - t1)/t3));
        start = 1 + ((seisStart - 1)*t3 - tmpStart + t1)/tmp.dt;
    end %endIf
        
    if (t2 > tmpEnd)
        seisEnd = int32(num1 - ceil((t2 - tmpEnd)/t3));
        stop = tmp.npts - (tmpEnd - t1 - seisEnd*t3)/tmp.dt;
    else
        seisEnd = int32(num1);
        stop = tmp.npts - (tmpEnd - t2)/tmp.dt;
    end %endIf 
    %fprintf ('seisInsert[INFO] RAW Data Range: %d %d\n',min(tmp.d),max(tmp.d));
    
    %Data Cleanup & Filter  
    tmp = srmean(tmp);
    %fprintf ('dt after mean[INFO] %d\n',tmp.dt);
    %fprintf ('seisInsert[INFO] DEMEANED Data Range: %d %d\n',min(tmp.d),max(tmp.d));
    tmp = seisFilterBW(ftype, f, tmp);
    %fprintf ('dt after filter[INFO] %d\n',tmp.dt);
    %fprintf ('seisInsert[INFO] FILTERED Data Range: %d %d\n',min(tmp.d),max(tmp.d));
    %endFilter
    
   %fprintf ('seisInsert[INFO] seisData(seisStart:seisEnd) = tmp.d(start:t3:stop), dt %f %f %f %f %f %d\n',seisStart,seisEnd,start,t3/tmp.dt,stop,tmp.dt);
   % seisData(int32(seisStart):int32(seisEnd)) = tmp.d(int32(start):int32(t3):int32(stop));
   %
   % this is to avoid sampling issue, always use the shortest length
   %
    if(length(int32(seisStart):int32(seisEnd)) > length(int32(start):int32(t3/tmp.dt):int32(stop)))
       %fprintf ('seisInsert[INFO] Data Range: ONE\n');
       count = int32(seisStart)-1;
       %fprintf ('seisInsert[INFO] int32(start):int32(t3/tmp.dt):int32(stop) = %d:%d:%d\n',int32(start),int32(t3/tmp.dt),int32(stop));
       for (i=int32(start):int32(t3/tmp.dt):int32(stop)) 
          count = count+1;
          seisData(count) = tmp.d(i);
        end; %end for
     elseif (length(int32(seisStart):int32(seisEnd)) < length(int32(start):int32(t3/tmp.dt):int32(stop)))
       %fprintf ('seisInsert[INFO] Data Range: TWO\n');
       count = int32(start)-int32(t3/tmp.dt);
       for (i=int32(seisStart):int32(seisEnd))
          count = count+int32(t3/tmp.dt);
          seisData(i) = tmp.d(count);
        end; %end for
     else
          %fprintf ('seisInsert[INFO] Data Range: THREE\n');
          seisData(int32(seisStart):int32(seisEnd)) = tmp.d(int32(start):int32(t3/tmp.dt):int32(stop));
     end;%end if-elseif-else
   %end; %end if
   %fprintf ('seisInsert[INFO] FINAL Data Range: %d %d\n',min(seisData),max(seisData));
end %endIf
